// 'use strict';
/* jshint esversion: 6 */
/* jshint multistr: true */
/* globals Ext, dss, _, hex2rgb, rgb2hsv, brightness2Percent, escape, unescape, devLog, DSS, addDataIdToElement, joinAndCapitalize */

const actionSelectorConsts = {
    UNDO_SCENE_SHIFT: 1000,
    SPECIAL_ACTION_SHIFT: 2000,
    RESET_MODE_SHIFT: 32,
    SECONDS_PER_MINUTE: 60,
    MINUTES_PER_HOUR: 60,
    SECONDS_PER_HOUR: 3600,
    SHOW_ALL_GRID_ITEMS_ID: -3,
    SEPARATOR_GRID_ITEM_ID: -1
};

const audioActionRepeatMode = {
    NEVER: 0,
    ONCE: 1,
    INDEFINITELY: 2
};

function ActionGridShowAllCheckbox (config) {
    this.id = actionSelectorConsts.SHOW_ALL_GRID_ITEMS_ID;
    this.name = _('show all scenes');
    this.reachable = true;
    this.checkable = true;
    this.checked = config.checked;
}
function ActionGridSeparator (config) {
    this.id = actionSelectorConsts.SEPARATOR_GRID_ITEM_ID;
    this.reachable = config.reachable;
    this.name = '-';
    this.passive = true;
}

const actionSelectorLvl1OptionIds = {
    ACTIVITY_IN_ROOM: 0,
    USER_GROUP: 7,
    SINGLE_DEVICE: 1,
    USER_DEFINED_ACTIONS: 2,
    USER_STATES: 8,
    CALL_URL: 3,
    ACCESS: 4,
    WEATHER: 5,
    SECURITY: 6,
    HEATING: 9,
    AUDIO: 10
};
const actionSelectorDeviceActionOptionIds = {
    OFF: 1,
    ON: 2,
    LOCAL_ON: 3,
    BLINK: 4,
    CALL_SCENE: 5,
    LEVEL_0_OF_3: 6,    // 0 - off
    LEVEL_1_OF_3: 7,    // 128
    LEVEL_2_OF_3: 8,    // 255 - on
    LEVEL_0_OF_4: 9,    // 0 - off
    LEVEL_1_OF_4: 10,   // 96
    LEVEL_2_OF_4: 11,   // 160
    LEVEL_3_OF_4: 12,   // 255 - on
    IMPULSE: 13,
    COLOR: 'color',
    COLOR_TEMPERATURE: 'colTemp'
};

function isImpulseCapableJoker (device) {
    return device.GroupMembership[0] == DSS.Groups.JOKER &&
        device.productID == 12488 &&
        (device.functionID == 33052 || device.functionID == 33232 || device.functionID == 33240);
}

function printDeviceIconCell (device) {
    return '<td style="padding-right:5px;height:16px;">' +
        '   <img src="' + dss.staticDataModel.getImageForDevice(device) + '">' +
        '</td>';
}
function printSceneIconCell (zoneId, groupId, sceneId, errorStyle) {
    return '<td style="' + (errorStyle || '') + 'padding-right:5px;height:16px;">' +
        dss.staticDataModel.getImageForScene(zoneId, groupId, sceneId) +
        '</td>';
}
function printUndoSceneIconCell (zoneId, groupId, sceneId, errorStyle) {
    return '<td style="' + (errorStyle || '') + 'padding-right:5px;height:16px;">' +
        dss.staticDataModel.getImageForScene(zoneId, groupId, sceneId, true) +
        '</td>';
}
function printPaddedCell (description, errorStyle) {
    return '<td style="' + (errorStyle || '') + 'padding-right:5px;height:16px;">' +
        (description || '') +
        '</td>';
}
function printDecriptionCell (description, errorStyle) {
    return '<td style="' + (errorStyle || '') + '">' +
        (description || '') +
        '</td>';
}
function makeIncompleteActivityPrintFunction (dataItem) {   // , errorStyle
    return function (errorStyle) {
        return '<span ' + dataItem + ' style="' + (errorStyle || '') + '">' +
            _('incomplete activity') +
            '</span>';
    };
}

function getClusterName (clusterIdAsString) {
    const clusters = dss.staticDataModel.getUsergroups();
    const clusterId = parseInt(clusterIdAsString, 10);
    for (var i = 0; i < clusters.length; i++) {
        var cluster = clusters[i];
        if (cluster.id === clusterId) {
            return cluster.name;
        }
    }
    return Ext.String.format2(_('Group {group}'), { group: clusterId });
}

function fillAndAmendSceneStoreWithAllOff (config) {
    let maxLabelWidth = config.initialMaxLabelWidth || 0;
    config.sceneStore.each(function (record) {
        var newRecord = record.copy();
        if (newRecord.get('reachable') || config.allScenesVisible) {
            if (newRecord.get('id') < config.sceneIdLimit) {
                config.storeToFillAndAmend.add(newRecord);
                maxLabelWidth = Math.max(maxLabelWidth, config.calcWidth && config.calcWidth(newRecord.get('name')) || 0);
                if (newRecord.get('id') == DSS.Scenes.OFF) {
                    if (config.groupID == DSS.Groups.SHADOW) {
                        config.storeToFillAndAmend.add({
                            id: DSS.Scenes.OFF,
                            name: _('all closed'),
                            icon: newRecord.get('icon'),
                            force: true
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, config.calcWidth && config.calcWidth(_('all closed')) || 0);
                    } else if (config.excludedGroups.indexOf(config.groupID) === -1) {
                        config.storeToFillAndAmend.add({
                            id: DSS.Scenes.OFF,
                            name: _('all off'),
                            icon: newRecord.get('icon'),
                            force: true
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, config.calcWidth && config.calcWidth(_('all off')) || 0);
                    }
                }
            }
        }
    });
    return maxLabelWidth;
}

Ext.define(
    'DSS.component.ColorTemperatureDialog', {
        extend: 'Ext.window.Window',
        xtype: 'colortempdialog',

        mixins: {
            observable: 'Ext.util.Observable'
        },

        title: 'Select color temperature',

        width: 600,
        height: 200,

        layout: {
            type: 'vbox',
            align: 'stretch'
        },

        modal: true,
        closeAction: 'hide',

        buttons: [
            {
                handler: function () {
                    var pdlg = this.findParentByType('colortempdialog');
                    if (pdlg.isValid()) {
                        var temperature = pdlg.colortempNumber.getValue();
                        var brightness = pdlg.brightnessNumber.getValue();
                        pdlg.fireEvent('setValue', temperature, brightness);
                        pdlg.hide();
                    }
                }
            },
            {
                handler: function () {
                    var pdlg = this.findParentByType('colortempdialog');
                    pdlg.hide();
                }
            }
        ],

        items: [
            {
                xtype: 'container',
                layout: {
                    type: 'hbox',
                    align: 'middle'
                },
                flex: 4,
                padding: 5,
                items: [
                    {
                        xtype: 'slider',
                        name: 'colortempSlider',
                        align: 'middle',
                        margin: '0 5 0 5',
                        flex: 3,
                        increment: 100,
                        width: 200,
                        minValue: 2000,
                        maxValue: 6400,
                        value: 2000,
                        vertical: false
                    },
                    {
                        xtype: 'numberfield',
                        name: 'colortempNumber',
                        align: 'middle',
                        margin: '0 5 0 5',
                        flex: 0,
                        minValue: 2000,
                        maxValue: 6400,
                        value: 2000,
                        decimalPrecision: 0,
                        enableKeyEvents: true,
                        maxWidth: 70
                    },
                    {
                        xtype: 'displayfield',
                        value: 'K',
                        margin: '0 5 0 0'
                    }
                ]
            },
            {
                xtype: 'container',
                layout: {
                    type: 'hbox',
                    align: 'middle'
                },
                flex: 4,
                padding: 5,
                items: [
                    {
                        xtype: 'slider',
                        name: 'brightnessSlider',
                        align: 'middle',
                        margin: '0 5 0 5',
                        flex: 3,
                        increment: 1,
                        width: 200,
                        minValue: 0,
                        maxValue: 100,
                        value: 100,
                        vertical: false
                    },
                    {
                        xtype: 'numberfield',
                        name: 'brightnessNumber',
                        align: 'middle',
                        margin: '0 5 0 5',
                        flex: 0,
                        minValue: 0,
                        maxValue: 100,
                        value: 100,
                        decimalPrecision: 0,
                        enableKeyEvents: true,
                        maxWidth: 70
                    },
                    {
                        xtype: 'displayfield',
                        value: '%',
                        margin: '0 5 0 0'
                    }
                ]
            }
        ],

        colortempSlider: null,
        colortempNumber: null,
        brightnessSlider: null,
        brightnessNumber: null,
        percent2Brightness: function (val) {
            return Math.round(val * 2.551);
        },
        isValid: function () {
            return this.colortempNumber.isValid() && this.brightnessNumber.isValid();
        },
        setValue: function (colorTemp, bright) {
            this.colortempSlider.setValue(colorTemp);
            this.colortempNumber.setValue(colorTemp);

            this.brightnessSlider.setValue(bright);
            this.brightnessNumber.setValue(bright);

        },
        constructor: function (config) {
            this.callParent(arguments);

            var me = this;

            this.colortempSlider = this.query('[name=colortempSlider]')[0];
            this.colortempNumber = this.query('[name=colortempNumber]')[0];
            this.brightnessSlider = this.query('[name=brightnessSlider]')[0];
            this.brightnessNumber = this.query('[name=brightnessNumber]')[0];

            // field labels need to be set here, because otherwise, they
            // will be set before the translation kicks in
            this.colortempSlider.fieldLabel = _('Color temperature');
            this.brightnessSlider.fieldLabel = _('Brightness');

            if (config.colortemp !== undefined) {
                me.colortempSlider.setValue(config.colortemp);
                me.colortempNumber.setValue(config.colortemp);
            }

            if (config.brightness !== undefined) {
                me.brightnessSlider.setValue(brightness2Percent(config.brightness));
                me.brightnessNumber.setValue(brightness2Percent(config.brightness));
            }

            this.colortempSlider.on('changecomplete', function (slider, newValue) {     // , thumb, eOpts
                me.colortempNumber.setValue(newValue);
                var tempValue = dss.kelvinToMired(newValue);
                tempValue = dss.miredToMOCValue(tempValue);
                me.fireEvent('change', tempValue, me.percent2Brightness(me.brightnessNumber.getValue()));
            });
            this.colortempNumber.on('specialkey', function (field, e) {
                if (e.getKey() == e.ENTER) {
                    me.colortempSlider.setValue(me.colortempNumber.getValue());
                    var tempValue = dss.kelvinToMired(me.colortempNumber.getValue());
                    tempValue = dss.miredToMOCValue(tempValue);
                    me.fireEvent('change', tempValue, me.percent2Brightness(me.brightnessNumber.getValue()));
                }
            });
            this.colortempNumber.on('spin', function (field, dir) {     // , opts
                // spin events always happen *before* the value is changed, so
                // the value needs to be adjusted to reflect the new state
                // (instead of the old one)
                var newValue = me.colortempNumber.getValue() + me.colortempNumber.step * (dir === 'up' ? 1 : -1);
                me.colortempSlider.setValue(newValue);
                var tempValue = dss.kelvinToMired(newValue);
                tempValue = dss.miredToMOCValue(tempValue);
                me.fireEvent('change', tempValue, me.percent2Brightness(me.brightnessNumber.getValue()));
            });

            this.brightnessSlider.on('changecomplete', function (slider, newValue) {    // , thumb, eOpts
                me.brightnessNumber.setValue(newValue);
                var tempValue = dss.kelvinToMired(me.colortempNumber.getValue());
                tempValue = dss.miredToMOCValue(tempValue);
                me.fireEvent('change', tempValue, me.percent2Brightness(newValue));
            });
            this.brightnessNumber.on('keypress', function (field, e) {
                if (e.getKey() == e.ENTER) {
                    me.brightnessSlider.setValue(me.brightnessNumber.getValue());

                    var tempValue = dss.kelvinToMired(me.colortempNumber.getValue());
                    tempValue = dss.miredToMOCValue(tempValue);
                    me.fireEvent('change', tempValue, me.percent2Brightness(me.brightnessNumber.getValue()));
                }
            });
            this.brightnessNumber.on('spin', function (field, dir) {    // , opts
                // spin events always happen *before* the value is changed, so
                // the value needs to be adjusted to reflect the new state
                // (instead of the old one)
                var newValue = me.brightnessNumber.getValue() + me.colortempNumber.step * (dir === 'up' ? 1 : -1);
                me.brightnessSlider.setValue(newValue);
                var tempValue = dss.kelvinToMired(me.colortempNumber.getValue());
                tempValue = dss.miredToMOCValue(tempValue);
                me.fireEvent('change', tempValue, me.percent2Brightness(newValue));
            });

            this.addEvents('setValue');
            this.addEvents('change');
            this.setTitle(_('Select color temperature'));
            this.dockedItems.items[0].items.items[0].setText(_('OK'));
            this.dockedItems.items[0].items.items[1].setText(_('cancel'));
        }
    }
);


Ext.define(
    'DSS.component.ColorPicker', {
        extend: 'Ext.Component',
        alias: 'ColorPicker',

        mixins: {
            observable: 'Ext.util.Observable'
        },

        initialColorValue: '000000',

        clickable: true,

        border: 0,
        style: {
            borderStyle: 'inset',
            width: '2.5em',
            height: '2.5em'
        },

        autoEl: {
            tag: 'div',
            cls: 'hue-colorpicker-class'
        },

        jPickerClientPath: '/framework/jpicker/images/',

        getJPickerLocalization: function () {
            return {
                text: {
                    title: _('Drag Markers To Pick A Color'),
                    newColor: _('new'),
                    currentColor: _('current'),
                    ok: _('OK'),
                    cancel: _('cancel')
                },
                tooltips: {
                    colors: {
                        newColor: _('New Color - Press OK To Commit'),
                        currentColor: _('Click To Revert To Original Color')
                    },
                    buttons: {
                        ok: _('Commit To This Color Selection'),
                        cancel: _('Cancel And Revert To Original Color')
                    },
                    hue: {
                        radio: _('Set To Hue Color Mode'),
                        textbox: _('Enter A Hue Value (0-360°)')
                    },
                    saturation: {
                        radio: _('Set To Saturation Color Mode'),
                        textbox: _('Enter A Saturation Value (0-100%)')
                    },
                    value: {
                        radio: _('Set To Value Color Mode'),
                        textbox: _('Enter A Value Value (0-100%)')
                    },
                    red: {
                        radio: _('Set To Red Color Mode'),
                        textbox: _('Enter A Red Value (0-255)')
                    },
                    green: {
                        radio: _('Set To Green Color Mode'),
                        textbox: _('Enter A Green Value (0-255)')
                    },
                    blue: {
                        radio: _('Set To Blue Color Mode'),
                        textbox: _('Enter A Blue Value (0-255)')
                    },
                    alpha: {
                        radio: _('Set To Alpha Color Mode'),
                        textbox: _('Enter A Alpha Value (0-100)')
                    },
                    hex: {
                        textbox: _('Enter A Hex Color Value (#000000-#ffffff)'),
                        alpha: _('Enter A Alpha Value (#00-#ff)')
                    }
                }
            };
        },

        colorValue: null,
        ctmode: false,
        timeoutID: null,

        applyColorCSSBkImage: function (colval, el) {
            var rgb = hex2rgb(colval);
            var rgbstr = String(rgb[0]) + ',' + String(rgb[1]) + ',' + String(rgb[2]);
            var cssstr = Ext.String.format('radial-gradient(1.25em 1.25em, circle closest-side, rgba({0},255) 0%, rgba({0},255) 80%, rgba(100,100,100,255) 90%, rgba(100,100,100,0) 100%)', rgbstr);
            cssstr = Ext.String.format('background-image: -moz-{0}; background-image: -webkit-{0}; background-image: {0};', cssstr);
            el.dom.style.cssText += cssstr;
        },

        fillInnerHTML: function (ctmode) {
            try {
                var el = this.getEl();
                if (el !== undefined) {
                    var bkcolor = this.isDisabled()
                        ? '#888888'
                        : (ctmode ? '#FFFFFF' : this.getValue());
                    this.applyColorCSSBkImage(bkcolor, el);
                }
            } catch (e) {
                devLog && devLog(e);
            }
        },

        setCTMode: function (ctmode) {
            this.ctmode = ctmode;
            this.fillInnerHTML(ctmode);
        },

        setValue: function (color) {
            if (color[0] === '#') {
                this.colorValue = color.substr(1);
            } else {
                this.colorValue = color;
            }

            var el = this.getEl();
            if (el !== undefined) {
                this.applyColorCSSBkImage(this.ctmode ? '#FFFFFF' : this.getValue(), el);
            }
        },

        getValue: function () {
            return '#' + this.colorValue;
        },

        onRender: function () {
            this.callParent(arguments);

            this.fillInnerHTML(this.ctmode);

            if (this.clickable) {
                var me = this;
                this.getEl().on('click', function () {  // e, t, eOpts
                    me.openColorPickerWindow();
                });
            }
        },

        openColorPickerWindow: function () {
            var m = new Ext.Window({
                title: _('Select color'),
                width: 560,
                height: 350,
                modal: true
            });

            var me = this;

            var color2color = function (color) {
                var all = color.val('all');
                return all && '#' + all.hex || 'transparent';
            };

            var ccancel = function (color) {
                if (me.timeoutID !== null) {
                    window.clearTimeout(me.timeoutID);
                    me.timeoutID = null;
                }

                me.fireEvent('change', color2color(color));

                m.destroy();
            };

            var ccommit = function (color) {
                if (me.timeoutID !== null) {
                    window.clearTimeout(me.timeoutID);
                    me.timeoutID = null;
                }

                var oldColorValue = me.colorValue;
                me.setValue(color2color(color));
                m.destroy();
                me.fireEvent('setValue', '#' + me.colorValue, '#' + oldColorValue);
            };

            var clive = function (color) {
                if (me.timeoutID !== null) {
                    window.clearTimeout(me.timeoutID);
                    me.timeoutID = null;
                }

                // Do not send change event immediately; instead, wait for 500 milliseconds
                // if the clive callback is not invoked in these 500 milliseconds, the
                // change event is triggered
                // This makes sure the change event is not triggered all the time
                // (it would otherwise be fired hundreds of times per second..)
                me.timeoutID = window.setTimeout(
                    function () {
                        me.fireEvent('change', color2color(color));
                    },
                    500
                );

            };

            var color = this.getValue();

            m.show();

            $(m.body.dom).jPicker(
                {
                    color: {
                        active: new $.jPicker.Color({ hex: color })
                    },
                    images: {
                        clientPath: this.jPickerClientPath
                    },
                    localization: this.getJPickerLocalization()
                },
                ccommit, clive, ccancel
            );
        },

        constructor: function (config) {
            if (config.initialColorValue !== undefined) {
                this.initialColorValue = config.initialColorValue;
            }
            if (config.clickable !== undefined) {
                this.clickable = !!(config.clickable);
            }

            this.colorValue = this.initialColorValue;

            this.callParent(arguments);
            this.mixins.observable.constructor.call(this, config);

            this.addEvents('setValue');
            this.addEvents('change');
        }
    }
);


Ext.define('DSS.component.ActionConfigurator', {
    extend: 'DSS.component.Base',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    height: '100%',
    layout: 'border',
    selectedElement: null,
    gridStore: null,
    grids: {},
    mainGrid: null,
    macPanel: null,
    lblSelectorAnzeige: null,
    iDefaultWidth: 150,
    iDefaultHeight: 293,
    oButtonNew: null,
    oButtonEdit: null,
    oButtonUp: null,
    oButtonDown: null,
    oButtonDelete: null,
    oActualValue: null,
    fDelayEnabled: false,
    fAllowNoAction: false,
    fEnableUsergroup: false,
    fHighLevel: true,
    fIgnorePresence: false,
    fHeatingOperationMode: false,
    fHeatingExtendedOperationMode: false,
    fUMRExtendetFunctions: false,
    fUseAcutalWidth: false,
    oColorEditor: null,
    oColTempEditor: null,
    selectedColor: null,
    selectedColTemp: null,
    compileToUDA: function () {

    },
    updatePanelDescription: function () {
        this.fireEvent('updatePanelDescription');
    },
    getDescriptionString: function (oAction) {
        var sString = '';
        var sTitle = '';
        var oDevice;
        var sKey;

        switch (oAction.type) {

            case DSS.ActionTypes.DEVICE_MOC_VALUE: {
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    break;
                }

                sString = oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                const hexColor = dss.mocToHexColor(oAction.params);
                if (hexColor) {
                    sTitle = _('color selection');
                }
                const color = dss.mocToBrightnessTemperatureColor(oAction.params);
                if (color) {
                    sTitle = _('colortemperature selection');
                }
                sString += ': ' + sTitle;
                break;
            }

            case DSS.ActionTypes.DEVICE_ACTION: {
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    break;
                }

                sString += oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                sTitle = oAction.params.id;

                var actions = dss.staticDataModel.getStandardActions(oAction.params.dsuid);
                for (sKey in actions) {
                    if (sKey == oAction.params.id) {
                        sTitle = actions[sKey].title;
                        break;
                    }
                }
                actions = dss.staticDataModel.getDeviceActions(oAction.params.dsuid);
                for (sKey in actions) {
                    if (sKey == oAction.params.id) {
                        sTitle = actions[sKey].title;
                        break;
                    }
                }
                actions = dss.staticDataModel.getCustomActions(oAction.params.dsuid);
                for (sKey in actions) {
                    if (sKey == oAction.params.id) {
                        sTitle = actions[sKey].title;
                        break;
                    }
                }
                sString += ': ' + sTitle;
                break;
            }

            case DSS.ActionTypes.HEATING_MODE: {
                switch (oAction.params.mode) {
                    case 1:
                        sString += _('COMFORT Mode');
                        break;
                    case 2:
                        sString += _('ECO Mode');
                        break;
                    case 3:
                        sString += (oAction.params.reset ? _('reset NotUsed Mode') : _('NotUsed Mode'));
                        break;
                    case 4:
                        sString += (oAction.params.reset ? _('reset NIGHT Mode') : _('NIGHT Mode'));
                        break;
                    case 5:
                        sString += (oAction.params.reset ? _('reset HOLIDAY Mode') : _('HOLIDAY Mode'));
                        break;
                    case 6:
                        sString += (oAction.params.reset ? _('reset COOLING Mode') : _('COOLING Mode'));
                        break;
                    default:
                        sString += '-';
                        break;
                }
                sString += ' ' + _('for') + ' ';
                sString += _('Heating Modes');

                if (oAction.params.zone != 0) {
                    sString += ' ' + _('in') + ' ' + dss.staticDataModel.getNameForZone(oAction.params.zone);
                }
                break;
            }

            case DSS.ActionTypes.UNDO_ZONE_SCENE: {
                sString += _('Undo ');
                // no 'break' here on purpose
            }

            case DSS.ActionTypes.ZONE_SCENE: {
                if (oAction.params.force) {
                    sString += _('all ');
                }

                sString += dss.staticDataModel.getNameForScene(oAction.params.zone, oAction.params.group, oAction.params.scene);

                if (oAction.params.group != DSS.Groups.ALL) {
                    sString += ' ' + _('for') + ' ' + dss.staticDataModel.getNameForGroup(oAction.params.zone, oAction.params.group);
                }
                if (oAction.params.zone != DSS.Zones.ALL_DEVICES) {
                    sString += ' ' + _('in') + ' ' + dss.staticDataModel.getNameForZone(oAction.params.zone);
                } else if (oAction.params.scene < DSS.Scenes.AUTO_STANDBY && !dss.staticDataModel.isClusterGroup(oAction.params.group)) {
                    sString += ' ' + _('in all rooms');
                }
                break;
            }

            case DSS.ActionTypes.DEVICE_BLINK: {
                oDevice = dss.staticDataModel.getDevice(this.oActualValue.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    break;
                }

                sString += oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                sString += ' ' + _('blinking');
                break;
            }

            case DSS.ActionTypes.ZONE_BLINK: {
                sString += dss.staticDataModel.getNameForGroup(oAction.params.zone, oAction.params.group);
                if (oAction.params.zone != DSS.Zones.ALL_DEVICES) {
                    sString += ' ' + _('in') + ' ' + dss.staticDataModel.getNameForZone(oAction.params.zone);
                } else if (oAction.params.scene < DSS.Scenes.AUTO_STANDBY) {
                    sString += ' ' + _('in all rooms');
                }
                sString += ' ' + _('blinking');
                break;
            }

            case DSS.ActionTypes.CUSTOM_EVENT: {
                sString += _('action') + ': ' + dss.staticDataModel.getHLENameFromId(oAction.params.event);
                break;
            }

            case DSS.ActionTypes.URL: {
                sString = 'URL: ' + unescape(oAction.params.url);
                break;
            }

            case DSS.ActionTypes.AUDIO_ACTIVITY: {
                const zones = (dss.staticDataModel.getZoneArray() || [])
                    .reduce(function (zoneNamesById, zone) {
                        zoneNamesById[zone.ZoneID] = zone.name;
                        return zoneNamesById;
                    }, {});
                const activityZones = JSON.parse(oAction.params.zones).map(function (zoneId) {
                    return zones[zoneId]
                        ? zones[zoneId]
                        : ' #' + zoneId;
                }).join(', ');
                const activity = dss.staticDataModel.audioActivities.find(function (activity) {
                    return activity.id === oAction.params.contentSource;
                });
                const name = activity && activity.name || _('audio activity') + ' #' + oAction.params.contentSource;
                sString = [
                    Ext.String.format2(_('{name} in zones {zones}'), {
                        name: name,
                        zones: activityZones
                    }),
                    oAction.params.volume != null ? _('volume') + ': ' + oAction.params.volume : '',
                    oAction.params.repeatMode != null ? _('repeat') + ': ' + (oAction.params.repeatMode ? _('on') : _('off')) : '',
                    oAction.params.shuffleMode != null ? _('shuffle') + ': ' + (oAction.params.shuffleMode ? _('on') : _('off')) : ''
                ].filter(function (part) { return Boolean(part); }).join(', ');
                break;
            }
        }

        return sString;
    },
    _updatePanelDescription: function () {
        if (this.macPanel.collapsed) {
            this.macPanel.setTitle('');
            return;
        }
        var sStringTitle = _('edit activity:') + ' ' + _('incomplete activity');
        var sString;
        var oDevice;
        var sKey;
        if (this.oActualValue != null) {

            switch (this.oActualValue.type) {
                case DSS.ActionTypes.CHANGE_ADDON_STATE: {
                    const stateId = this.oActualValue.params.statename;
                    const userState = dss.staticDataModel.userStates && dss.staticDataModel.userStates.find(function (state) {
                        return state.name === stateId;
                    });
                    const userStateActive = this.oActualValue.params.state === 'active';
                    sString = Ext.String.format2('{state} is {stateValue}', {
                        state: userState ? userState.displayName : _('user state') + ' #' + stateId,
                        stateValue: userState ? userState[userStateActive ? 'setName' : 'resetName'] : (userStateActive ? _('active') : _('inactive'))
                    });
                    sStringTitle = _('edit activity:') + ' ' + sString;
                    break;
                }

                case DSS.ActionTypes.CHANGE_STATE: {
                    var splitState = this.oActualValue.params.statename.split('.');
                    if (splitState.length === 1) {
                        sString = dss.staticDataModel.getNameForState(this.oActualValue.params.statename, this.oActualValue.params.state);
                    } else {
                        sString = Ext.String.format2(_('{state} in {clustername}'), {
                            state: dss.staticDataModel.getNameForState(splitState[0], this.oActualValue.params.state),
                            clustername: getClusterName(splitState[1].substr(5))
                        });
                    }
                    sStringTitle = _('edit activity:') + ' ' + sString;
                    break;
                }

                case DSS.ActionTypes.DEVICE_MOC_VALUE: {
                    sString = this.getDescriptionString(this.oActualValue);
                    if (!sString) {
                        break;
                    }
                    sStringTitle = _('edit activity:') + ' ' + sString;
                    break;
                }

                case DSS.ActionTypes.DEVICE_ACTION:
                case DSS.ActionTypes.HEATING_MODE:
                case DSS.ActionTypes.ZONE_SCENE: // scene, group, zone, force
                case DSS.ActionTypes.UNDO_ZONE_SCENE: { // scene, group, zone, force
                    sStringTitle = _('edit activity:') + ' ' + this.getDescriptionString(this.oActualValue);
                    break;
                }

                case DSS.ActionTypes.DEVICE_SCENE: { // scene, dsuid, force
                    oDevice = dss.staticDataModel.getDevice(this.oActualValue.params.dsuid);
                    if (typeof oDevice.ZoneID === 'undefined') {
                        break;
                    }

                    sString = oDevice.name + ' ' + _('in') + ' ';
                    sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                    if (this.oActualValue.params.scene == DSS.Scenes.MIN) {
                        sString = sString + ' ' + _('turn off');
                    } else if (this.oActualValue.params.scene == DSS.Scenes.MAX) {
                        sString = sString + ' ' + _('turn on');
                    } else if (this.oActualValue.scene == DSS.Scenes.LOCAL_ON) {
                        sString = sString + ' ' + _('turn local on');
                    } else {
                        sString = dss.staticDataModel.getNameForScene(oDevice.ZoneID, oDevice.GroupID, this.oActualValue.params.scene) + ' ' + _('for') + ' ' + sString;
                        if (this.oActualValue.params.force) {
                            sString = _('all ') + sString;
                        }
                    }

                    if (oDevice.specialActions) {
                        if (oDevice.specialActions.actions) {
                            if (oDevice.specialActions.actions.deviceActions) {
                                for (sKey in oDevice.specialActions.actions.deviceActions) {
                                    if (oDevice.specialActions.actions.deviceActions[sKey].scene != null) {
                                        if (oDevice.specialActions.actions.deviceActions[sKey].scene.toString() == this.oActualValue.params.scene.toString()) {
                                            sString = oDevice.name + ' ' + _('in') + ' ';
                                            sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                                            sString += ': ' + oDevice.specialActions.actions.deviceActions[sKey].name;
                                        }
                                    }
                                }
                            }
                        }
                    }

                    sStringTitle = _('edit activity:') + ' ' + sString;
                    break;
                }

                case DSS.ActionTypes.DEVICE_VALUE: { // value, dsuid
                    sStringTitle = '<span style="color:#ff0000;">' + _('The activity is not longer supported. Please choose a other activity or delete it') + '</span>';
                    oDevice = dss.staticDataModel.getDevice(this.oActualValue.params.dsuid);
                    if (typeof oDevice.ZoneID === 'undefined') {
                        break;
                    }

                    if (oDevice.outputMode == 34) {
                        sStringTitle = oDevice.name + ' ' + _('in') + ' ';
                        sStringTitle += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                        if (this.oActualValue.params.value == 0) {  // aus
                            sStringTitle = sStringTitle + ' ' + _('turn off');
                        }
                        if (this.oActualValue.params.value == 128) {    // aus
                            sStringTitle = sStringTitle + ' ' + _('set to Stage 1');
                        }
                        if (this.oActualValue.params.value == 255) {    // aus
                            sStringTitle = sStringTitle + ' ' + _('set to Stage 2');
                        }

                    } else if (oDevice.outputMode == 38) {
                        sStringTitle = oDevice.name + ' ' + _('in') + ' ';
                        sStringTitle += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                        if (this.oActualValue.params.value == 0) {
                            sStringTitle = sStringTitle + ' ' + _('turn off');
                        }
                        if (this.oActualValue.params.value == 96) {  // aus
                            sStringTitle = sStringTitle + ' ' + _('set to Stage 1');
                        }
                        if (this.oActualValue.params.value == 160) { // aus
                            sStringTitle = sStringTitle + ' ' + _('set to Stage 2');
                        }
                        if (this.oActualValue.params.value == 255) { // aus
                            sStringTitle = sStringTitle + ' ' + _('set to Stage 3');
                        }
                    }
                    if (oDevice.specialActions) {
                        if (oDevice.specialActions.actions) {
                            if (oDevice.specialActions.actions.deviceActions) {
                                for (sKey in oDevice.specialActions.actions.deviceActions) {
                                    if (oDevice.specialActions.actions.deviceActions[sKey].outputValue != null) {
                                        if (oDevice.specialActions.actions.deviceActions[sKey].outputValue == this.oActualValue.params.value) {
                                            sStringTitle = oDevice.name + ' ' + _('in') + ' ';
                                            sStringTitle += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                                            sStringTitle += ': ' + oDevice.specialActions.actions.deviceActions[sKey].name;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.ZONE_VALUE: { // value, dsuid
                    sStringTitle = '<span style="color:#ff0000;">' + _('The activity is not longer supported. Please choose a other activity or delete it') + '</span>';
                    break;
                }

                case DSS.ActionTypes.DEVICE_BLINK: { // dsuid
                    sString = this.getDescriptionString(this.oActualValue);
                    if (!sString) {
                        break;
                    }
                    sStringTitle = _('edit activity:') + ' ' + sString;
                    break;
                }

                case DSS.ActionTypes.ZONE_BLINK: // group, zone
                case DSS.ActionTypes.CUSTOM_EVENT: // event
                case DSS.ActionTypes.URL: // url
                case DSS.ActionTypes.AUDIO_ACTIVITY: {
                    sStringTitle = _('edit activity:') + ' ' + this.getDescriptionString(this.oActualValue);
                    break;
                }
                default:
                    // sStringTitle = _('edit activity:') + ' ' + _('incomplete activity');
            }
        }
        this.macPanel.setTitle(sStringTitle);
    },
    setHeight: function (oHeight) {
        this.macPanel.setHeight(oHeight - 30);
        this.iDefaultHeight = oHeight - 74;
        for (var oKey in this.grids) {
            this.grids[oKey].setHeight(this.iDefaultHeight);
        }
        this.callParent(arguments);
    },
    defMacGridRenderer: function (val, metadata, record) {      // , colIdx, store, view
        const name = record.get('name');
        if (name == '-') {
            return '<table style="height:16px;width:100%;pointer-events:none;"><tr style="height:16px;"><td width="100%"><hr></td></tr></table>';
        }

        const icon = record.get('icon');
        const dSID = record.get('dSID');
        const dSUID = record.get('dSUID');
        const groupId = record.get('group');
        const id = record.get('id');
        const _id = record.get('_id');
        let fHasIcon = icon != null;
        // definition requires '>' button,
        // or it's a group == needs '>' button,
        // or it's a device == needs '>' button
        let fHasNext = Boolean(record.get('next')) || groupId != null || dSID != null || dSUID != null;
        const fIsCheckable = Boolean(record.get('checkable'));
        if (fIsCheckable) {
            fHasNext = false; // for checkboxes, no '>' button is needed
            fHasIcon = false; // checkbox items need no icon
        }

        const fIsPassive = Boolean(record.get('passive'));
        if (fIsPassive) {
            fHasNext = false;               // column disabled == no '>' button
            fIsChecked = false;             // column disabled == cannot be checked
        }

        var sReturn = '';
        const tableOpeningTag = '<table style="height:16px;width:100%;"' +
            (dSID ? ' data-dsid="' + dSID + '"' : '') +
            (dSUID ? ' data-dsuid="' + dSUID + '"' : '') +
            (groupId != null && groupId !== '' ? ' data-group-id="' + groupId + '"' : '') +
            (id != null && id !== '' ? ' data-id="' + id + '"' : (_id != null && _id !== '' ? ' data-id="' + _id + '"' : '')) +
            (name ? ' data-name="' + joinAndCapitalize(name) + '"' : '') +
            '>';
        if (id === actionSelectorDeviceActionOptionIds.COLOR_TEMPERATURE) {
            sReturn += '<tr><td rowspan="3"><img src="' + icon + '" style="margin-right: 5px;"></td>';
            if (fIsPassive) {
                sReturn += '<td style="color:#888888;" width="100%" colspan="2">' + name + '</td></tr>';
            } else {
                sReturn += '<td width="100%" colspan="2">' + name + '</td></tr>';
            }
            if (this.parent.selectedColTemperature != null) {
                return tableOpeningTag + sReturn +
                '<tr><td>' + _('ColTemperature') + ':</td><td id="colorTempDisplayTemperature">' + this.parent.selectedColTemperature.temperature + 'K</td></tr>' +
                '<tr><td>' + _('Brightness') + ':</td><td id="colorTempDisplayBrightness">' + this.parent.selectedColTemperature.bright + '%</td></tr></table>' +
                '</table>';
            } else {
                return tableOpeningTag + sReturn +
                '<tr><td>' + _('ColTemperature') + ':</td><td id="colorTempDisplayTemperature">-</td></tr>' +
                '<tr><td>' + _('Brightness') + ':</td><td id="colorTempDisplayBrightness">-</td></tr></table>' +
                '</table>';
            }
        }

        var iColspaceCounter = 4;
        if (fHasIcon) {
            iColspaceCounter--;
            sReturn += '<td><img src="' + icon + '" style="margin-right: 5px;"></td>';
        }
        if (fIsCheckable) {
            iColspaceCounter--;
            if (record.get('checked')) {
                sReturn += '<td style="padding-right: 5px;"><div style="width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif); background-position: 0px -13px;" /></td>';
            } else {
                sReturn += '<td style="padding-right: 5px;"><div style="width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif);" /></td>';
            }
        }
        if (fHasNext) {
            iColspaceCounter--;
        }
        if (id === actionSelectorDeviceActionOptionIds.COLOR) {
            iColspaceCounter--;
        }

        if (fIsPassive) {
            sReturn += '<td style="color:#888888;" colspan="' + iColspaceCounter + '" width="100%">' + name + '</td>';
        } else {
            sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + name + '</td>';
        }
        if (id === actionSelectorDeviceActionOptionIds.COLOR) {
            if (this.parent.selectedColor != null) {
                sReturn += '<td><div id="colorDisplay" style="width:24px;height:24px;border-radius:12px;border:1px solid #bbbbbb;background-color:' + this.parent.selectedColor + ';"/></td>';
            } else {
                sReturn += '<td><div id="colorDisplay" style="width:24px;height:24px;border-radius:12px;border:1px solid #bbbbbb;"/></td>';
            }
        }
        if (fHasNext) {
            sReturn += '<td>&gt;</td>';
        }

        return tableOpeningTag + '<tr>' + sReturn + '</tr></table>';
    },
    defMainGridRenderer: function (val, metadata, record) {     // , colIdx, store, view
        var oAction = record.get('action');
        oAction.params.type = oAction.type;
        var fValid = dss.validateActionObject(oAction.params);
        var sError = '';
        if (fValid == dss.validValue.completlyWrong) {
            sError = 'color:#ff0000;';
        }
        if (fValid == dss.validValue.temporaryWrong) {
            sError = 'color:#aaaaaa;';
        }

        var sString;
        var oDevice;
        var i;
        var sKey;
        var sName;

        const dataItem = 'data-item="' + (this.parent.actionSelectorName ? this.parent.actionSelectorName : '') + 'Item"';
        const printIncompleteActivity = makeIncompleteActivityPrintFunction(dataItem, sError);
        const wrapInTableRow = function (content) {
            return '<table ' + dataItem + '><tr>' + content + '</tr></table>';
        };

        switch (oAction.type) {

            case DSS.ActionTypes.DEVICE_MOC_VALUE: {
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    return printIncompleteActivity(sError);
                }

                sString = oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                const hexColor = dss.mocToHexColor(oAction.params);
                if (hexColor) {
                    return wrapInTableRow(
                        printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString, sError) +
                        '   <td style="padding-left:5px;">' +
                        '       <div style="width:24px;height:24px;border-radius:12px;border:1px solid #bbbbbb;background-color:' + hexColor + ';" />' +
                        '   </td>'
                    );
                }
                const color = dss.mocToBrightnessTemperatureColor(oAction.params);
                if (color) {
                    return wrapInTableRow(
                        printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString + ' ' +
                        _('ColTemperature') + ': ' + color.temperature + ' K ' +
                        _('Brightness') + ': ' + color.bright + ' %', sError) +
                        '<td style="padding-left:5px;"></td>'
                    );
                }
                break;
            }

            case DSS.ActionTypes.CHANGE_ADDON_STATE: {
                if (oAction.params.addonId === 'system-addon-user-defined-states') {
                    var stateName = oAction.params.statename;
                    var value = oAction.params.state;

                    var oUserStates = dss.staticDataModel.getUserStates();
                    var sStringValue;
                    var image = null;
                    if (oUserStates) {
                        for (i = 0; i < oUserStates.length; i++) {
                            if (oUserStates[i].name == stateName) {
                                if (value === 'active') {
                                    sStringValue = oUserStates[i].setName;
                                    image = dss.staticDataModel.baseImagePath + 'state/uds_state_active.png';
                                } else {
                                    sStringValue = oUserStates[i].resetName;
                                    image = dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png';
                                }
                                sStringValue = Ext.String.format2(_('Customstate {state} change to {value}'), {
                                    state: oUserStates[i].displayName,
                                    value: sStringValue
                                });
                            }
                        }
                    }

                    sString = wrapInTableRow(
                        (image != null
                            ? printPaddedCell('<img src="' + image + '">')
                            : printPaddedCell()
                        ) +
                        printDecriptionCell(sStringValue, sError)
                    );
                    return sString;
                // } else if (oAction.params['addonId'] === 'system-addon-heating-controller') {
                //     var stateName = oAction.params.statename;
                //     var value = oAction.params.state;
                //
                //     var image = null;
                //     if ([ 'off', 'heating', 'cooling' ].indexOf(value) > -1) {
                //         image = dss.staticDataModel.baseImagePath + 'state/heating_mode_' + value + '.png';
                //     }
                //
                //     var sString = rowTableTag + '<tr>';
                //     if (image != null) {
                //         sString += '<td style="' + sError + 'padding-right:5px;"><img src="' + image + '"></td>';
                //     } else {
                //         sString += '<td style="' + sError + 'padding-right:5px;"></td>';
                //     }
                //
                //     if (stateName === 'apartment_controller') {
                //         sString += '<td style="' + sError + '">' + _('heating mode ' + value);
                //     } else {
                //         sString += '<td style="' + sError + '"><b>' + _(stateName) + '</b> ' + _('change to') + ' <b>' + value + '</b>';
                //     }
                //
                //     sString += '</td></tr></table>';
                //     return sString;
                } else {
                    sString = wrapInTableRow(
                        printPaddedCell() +
                        printDecriptionCell('<b>' + oAction.params.statename + '</b> ' +
                            _('change to') + ' <b>' + oAction.params.state + '</b> ' +
                            _('for App') + ' <b>' + oAction.params.addonId + '</b> (' + _('internal') + ')', sError)
                    );
                    return sString;
                }
                break;
            }
            case DSS.ActionTypes.CHANGE_STATE: {
                var splitState = oAction.params.statename.split('.');
                sString = printPaddedCell('<img src="' + dss.staticDataModel.getImagePathForState(oAction.params.statename, oAction.params.state) + '">', sError);
                if (splitState.length === 1) {
                    sString += printDecriptionCell(dss.staticDataModel.getNameForState(oAction.params.statename, oAction.params.state), sError);
                } else {
                    sString += printDecriptionCell(Ext.String.format2(_('{state} in {clustername}'), {
                        state: dss.staticDataModel.getNameForState(splitState[0], oAction.params.state),
                        clustername: getClusterName(splitState[1].substr(5))
                    }), sError);
                }
                return wrapInTableRow(sString);
                break;
            }
            case DSS.ActionTypes.HEATING_MODE: {
                return wrapInTableRow(
                    printSceneIconCell(oAction.params.zone, DSS.Groups.TEMPERATURE, oAction.params.mode, sError) +
                    printDecriptionCell(this.parent.getDescriptionString(oAction), sError)
                );
            }
            case DSS.ActionTypes.ZONE_SCENE: { // scene, group,zone,force
                return wrapInTableRow(
                    printSceneIconCell(oAction.params.zone, oAction.params.group, oAction.params.scene, sError) +
                    printDecriptionCell(this.parent.getDescriptionString(oAction), sError)
                );
            }
            case DSS.ActionTypes.UNDO_ZONE_SCENE: { // scene, group,zone,force
                return wrapInTableRow(
                    printUndoSceneIconCell(oAction.params.zone, oAction.params.group, oAction.params.scene, sError) +
                    printDecriptionCell(this.parent.getDescriptionString(oAction), sError)
                );
            }
            case DSS.ActionTypes.ZONE_VALUE: { // value, dsuid
                sString = Math.round(oAction.params.value / 2.55) + '%' + ' ' + _('for') + ' ';
                if (oAction.params.group != 0) {
                    sString += ' ' + _('for') + ' ' + dss.staticDataModel.getNameForGroup(oAction.params.zone, oAction.params.group);
                }
                if (oAction.params.zone != 0) {
                    sString += ' ' + _('in') + ' ' + dss.staticDataModel.getNameForZone(oAction.params.zone);
                }
                return wrapInTableRow(
                    printDecriptionCell(sString, sError)
                );
            }
            case DSS.ActionTypes.DEVICE_ACTION: {
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                sString = this.parent.getDescriptionString(oAction);
                if (!sString) {
                    return printIncompleteActivity(sError);
                }
                return wrapInTableRow(
                    printDeviceIconCell(oDevice) +
                    printDecriptionCell(sString, sError)
                );
            }
            case DSS.ActionTypes.DEVICE_SCENE: { // scene, dsuid,force
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    return printIncompleteActivity(sError);
                }
                sString = oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                const specialActions = oDevice.specialActions.actions;
                if (specialActions && specialActions.deviceActions) {
                    for (sKey in specialActions.deviceActions) {
                        const action = specialActions.deviceActions[sKey];
                        sName = action.name;
                        if (action.scene != null && action.scene == oAction.params.scene) {
                            return wrapInTableRow(
                                printDeviceIconCell(oDevice) +
                                printDecriptionCell(sString + ' ' + sName, sError)
                            );
                        }
                    }
                }

                if (oAction.params.scene == DSS.Scenes.IMPULSE && isImpulseCapableJoker(oDevice)) {
                    sString = printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString + ' ' + _('executes impulse'), sError);
                } else if (oAction.params.scene == DSS.Scenes.MIN) {
                    sString = printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString + ' ' + _('turn off'), sError);
                } else if (oAction.params.scene == DSS.Scenes.MAX) {
                    sString = printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString + ' ' + _('turn on'), sError);
                } else if (oAction.params.scene == DSS.Scenes.LOCAL_ON) {
                    sString = printDeviceIconCell(oDevice) +
                        printDecriptionCell(sString + ' ' + _('turn on'), sError);
                } else {
                    sString = dss.staticDataModel.getNameForScene(oDevice.ZoneID, oDevice.GroupID, oAction.params.scene) + ' ' + _('for') + ' ' + sString;
                    if (oAction.params.force) {
                        sString = _('all ') + sString;
                    }
                    sString = printSceneIconCell(oDevice.ZoneID, oDevice.GroupID, oAction.params.scene, sError) +
                        printDecriptionCell(sString);
                }
                return wrapInTableRow(sString);
                break;
            }
            case DSS.ActionTypes.DEVICE_VALUE: { // value, dsuid
                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                if (typeof oDevice.ZoneID === 'undefined') {
                    return printIncompleteActivity(sError);
                }

                sString = oDevice.name + ' ' + _('in') + ' ';
                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);

                if (oDevice.specialActions.actions) {
                    if (oDevice.specialActions.actions.deviceActions) {
                        for (sKey in oDevice.specialActions.actions.deviceActions) {
                            sName = oDevice.specialActions.actions.deviceActions[sKey].name;
                            if (oDevice.specialActions.actions.deviceActions[sKey].outputValue != null) {
                                if (oDevice.specialActions.actions.deviceActions[sKey].outputValue == oAction.params.value) {
                                    return wrapInTableRow(
                                        printDeviceIconCell(oDevice) +
                                        printDecriptionCell(sString + ' ' + sName, sError)
                                    );
                                }
                            }
                        }
                    }
                }

                if (oDevice.outputMode == 34) {
                    sString = oDevice.name + ' ' + _('in') + ' ';
                    sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                    if (oAction.params.value == 0) { // aus
                        sString = sString + ' ' + _('turn off');
                    }
                    if (oAction.params.value == 128) { // aus
                        sString = sString + ' ' + _('set to Stage 1');
                    }
                    if (oAction.params.value == 255) { // aus
                        sString = sString + ' ' + _('set to Stage 2');
                    }
                    return sString;
                } else if (oDevice.outputMode == 38) {
                    sString = oDevice.name + ' ' + _('in') + ' ';
                    sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                    if (oAction.params.value == 0) { // aus
                        sString = sString + ' ' + _('turn off');
                    }
                    if (oAction.params.value == 96) { // aus
                        sString = sString + ' ' + _('set to Stage 1');
                    }
                    if (oAction.params.value == 160) { // aus
                        sString = sString + ' ' + _('set to Stage 2');
                    }
                    if (oAction.params.value == 255) { // aus
                        sString = sString + ' ' + _('set to Stage 3');
                    }
                    return sString;
                }

                return wrapInTableRow(
                    printDecriptionCell(Math.round(oAction.params.value / 2.55) + '%' + ' ' + _('for') + ' ' +
                        oDevice.name + ' ' + _('in') + ' ' +
                        dss.staticDataModel.getNameForZone(oDevice.ZoneID), sError + 'height:16px;')
                );
            }
            case DSS.ActionTypes.DEVICE_BLINK: { // dsuid
                var sString = this.parent.getDescriptionString(oAction);
                if (!sString) {
                    return printIncompleteActivity(sError);
                }

                oDevice = dss.staticDataModel.getDevice(oAction.params.dsuid);
                return wrapInTableRow(
                    printDeviceIconCell(oDevice) +
                    printDecriptionCell(sString, sError)
                );
            }
            case DSS.ActionTypes.ZONE_BLINK: { // group,zone
                return wrapInTableRow(
                    printPaddedCell('<img src="' + dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png">', sError) +
                    printDecriptionCell(this.parent.getDescriptionString(oAction), sError)
                );
            }
            case DSS.ActionTypes.CUSTOM_EVENT: { // event
                return '<span ' + dataItem + ' style="' + sError + '">' + this.parent.getDescriptionString(oAction) + '</span>';
            }
            case DSS.ActionTypes.URL: { // url
                return '<span ' + dataItem + ' style="' + sError + '">' + this.parent.getDescriptionString(oAction) + '</span>';
            }
            case DSS.ActionTypes.AUDIO_ACTIVITY: {
                return '<span ' + dataItem + ' style="' + sError + '">' + this.parent.getDescriptionString(oAction) + '</span>';
            }
            default: {
                return printIncompleteActivity('color:#ff0000;');
            }
        }
    },
    initStores: function (fHiglevelShown, fUserstateShown) {
        var activities;
        var i;
        if (Ext.data.StoreManager.lookup('actionConfigurator_userStateStore') == null) {

            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_userStateStore',
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var oUserStates = dss.staticDataModel.getUserStates();

            if (oUserStates) {
                for (i = 0; i < oUserStates.length; i++) {
                    if (oUserStates[i].type === 'custom-states') {
                        activities = [{
                            name: oUserStates[i].displayName,
                            _id: oUserStates[i].name,
                            next: true,
                            icon: null
                        }];
                        Ext.data.StoreManager.lookup('actionConfigurator_userStateStore').add(activities);
                    }
                }
            }
        }

        if (Ext.data.StoreManager.lookup('actionConfigurator_audioActivityStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_audioActivityStore',
                fields: [ '_id', 'vdcDsuid', 'icon', 'name', 'repeat', 'shuffle', 'volume', 'restart', 'zones', 'notification' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            dss.staticDataModel.getAudioActivities();
            if (dss.staticDataModel.audioActivities) {
                Ext.data.StoreManager.lookup('actionConfigurator_audioActivityStore').add(
                    dss.staticDataModel.audioActivities.map(function (activity) {
                        const storeItem = Object.assign({}, activity);
                        storeItem._id = activity.id;
                        storeItem.icon = dss.staticDataModel.getImagePathForAudioActivity(activity.whatToPlay.type);
                        return storeItem;
                    })
                );
            }
        }

        if (Ext.data.StoreManager.lookup('actionConfigurator_accessStore') == null) {
            activities = [ {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.DOORBELL),
                id: DSS.Scenes.DOORBELL,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.DOORBELL)
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ABSENT),
                id: DSS.Scenes.ABSENT,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ABSENT)
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PRESENT),
                id: DSS.Scenes.PRESENT,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PRESENT)
            } ];
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_accessStore',
                fields: [ 'id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('actionConfigurator_accessStore').add(activities);
        }

        // if (Ext.data.StoreManager.lookup('actionConfigurator_heatingSystemStore') == null) {
        //     activities = [{
        //         name: dss.staticDataModel.getNameForState('heating_mode', 'off'),
        //         id : "heating_mode.off",
        //         next: false,
        //         icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF)
        //     }];
        //
        //
        //     if (dss.staticDataModel.ClimateControllerSettingsCache!=null) {
        //         if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities!=null){
        //             if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.coolingSupported!=null)
        //                 if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.coolingSupported) {
        //                     activities.push({
        //                         name: dss.staticDataModel.getNameForState('heating_mode', 'cooling'),
        //                         id : "heating_mode.cooling",
        //                         next: false,
        //                         icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.USER1)
        //                     });
        //                 }
        //             if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.heatingSupported!=null)
        //                 if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.heatingSupported) {
        //                     activities.push({
        //                         name: dss.staticDataModel.getNameForState('heating_mode', 'heating'),
        //                         id : "heating_mode.heating",
        //                         next: false,
        //                         icon: dss.staticDataModel.getImagePathForScene(0, 3, DSS.Scenes.USER1)
        //                     });
        //                 }
        //         }
        //     }
        //
        //
        //     Ext.create('Ext.data.Store', {
        //         storeId : 'actionConfigurator_heatingSystemStore',
        //         fields : [ 'id', 'name', 'next', 'icon' ],
        //         proxy : {
        //             type: 'memory',
        //             reader : {
        //                 type: 'json',
        //                 root : 'items'
        //             }
        //         }
        //     });
        //     Ext.data.StoreManager.lookup('actionConfigurator_heatingSystemStore').add(activities);
        // }

        if (Ext.data.StoreManager.lookup('actionConfigurator_weatherStore') == null) {
            activities = [ {
                name: dss.staticDataModel.getNameForState('rain', 'active'),
                id: 'rain_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('rain', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('rain', 'inactive'),
                id: 'rain_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('rain', 'inactive')
            }, {
                name: dss.staticDataModel.getNameForState('wind', 'active'),
                id: 'wind_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('wind', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('wind', 'inactive'),
                id: 'wind_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('wind', 'inactive')
            } ];

            var states = dss.staticDataModel.getStateArray();
            for (var stateKey in states) {
                if (states.hasOwnProperty(stateKey)) {
                    var state = states[stateKey];
                    if (state.name.substring(0, 10) === 'wind.group') {
                        const clusterName = getClusterName(state.name.substring(10));
                        activities.push({
                            name: Ext.String.format1(_('{state} in {clustername}'), {
                                state: dss.staticDataModel.getNameForState('wind', 'active'),
                                clustername: clusterName
                            }),
                            id: state.name + '_active',
                            next: false,
                            icon: dss.staticDataModel.getImagePathForState('wind', 'active')
                        });
                        activities.push({
                            name: Ext.String.format1(_('{state} in {clustername}'), {
                                state: dss.staticDataModel.getNameForState('wind', 'inactive'),
                                clustername: clusterName
                            }),
                            id: state.name + '_inactive',
                            next: false,
                            icon: dss.staticDataModel.getImagePathForState('wind', 'inactive')
                        });
                    }
                }
            }

            activities.push({
                name: dss.staticDataModel.getNameForState('hail', 'active'),
                id: 'hail_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('hail', 'active')
            });
            activities.push({
                name: dss.staticDataModel.getNameForState('hail', 'inactive'),
                id: 'hail_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('hail', 'inactive')
            });

            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_weatherStore',
                fields: [ 'id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('actionConfigurator_weatherStore').add(activities);
        }

        if (Ext.data.StoreManager.lookup('actionConfigurator_securityStore') == null) {
            activities = [ {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PANIC),
                id: DSS.Scenes.PANIC,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PANIC)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PANIC),
                id: actionSelectorConsts.UNDO_SCENE_SHIFT + DSS.Scenes.PANIC,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.PANIC, true)
            }, {
                name: dss.staticDataModel.getNameForState('fire', 'active'),
                id: 'fire_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('fire', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('fire', 'inactive'),
                id: 'fire_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('fire', 'inactive')
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_1),
                id: DSS.Scenes.ALARM_1,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_1)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_1),
                id: actionSelectorConsts.UNDO_SCENE_SHIFT + DSS.Scenes.ALARM_1,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_1, true)
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_2),
                id: DSS.Scenes.ALARM_2,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_2)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_2),
                id: actionSelectorConsts.UNDO_SCENE_SHIFT + DSS.Scenes.ALARM_2,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_2, true)
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_3),
                id: DSS.Scenes.ALARM_3,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_3)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_3),
                id: actionSelectorConsts.UNDO_SCENE_SHIFT + DSS.Scenes.ALARM_3,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_3, true)
            }, {
                name: dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_4),
                id: DSS.Scenes.ALARM_4,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_4)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_4),
                id: actionSelectorConsts.UNDO_SCENE_SHIFT + DSS.Scenes.ALARM_4,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.ALL, DSS.Scenes.ALARM_4, true)
            } ];
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_securityStore',
                fields: [ 'id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('actionConfigurator_securityStore').add(activities);
        }

        if (Ext.data.StoreManager.lookup('actionConfigurator_l1Store') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_l1Store',
                fields: [ '_id', 'name', 'next', 'icon', 'passive' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            activities = [ {
                name: _('activity in room'),
                _id: actionSelectorLvl1OptionIds.ACTIVITY_IN_ROOM,
                next: true,
                icon: null,
                passive: dss.staticDataModel.getZoneArray().length === 1
            }];
            Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add(activities);

            if (this.fEnableUsergroup) {
                Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                    name: _('usergroup'),
                    _id: actionSelectorLvl1OptionIds.USER_GROUP,
                    next: true,
                    icon: null,
                    passive: dss.staticDataModel.getUsergroups().length === 0
                }]);
            }

            Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                name: _('single device'),
                _id: actionSelectorLvl1OptionIds.SINGLE_DEVICE,
                next: true,
                icon: null,
                passive: dss.staticDataModel.getZoneArray().length === 1
            }]);

            if (fHiglevelShown) {
                Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                    name: _('user def. actions'),
                    _id: actionSelectorLvl1OptionIds.USER_DEFINED_ACTIONS,
                    next: true,
                    icon: null,
                    passive: dss.staticDataModel.getHLE().length === 0
                }]);
            }
            if (fUserstateShown) {
                Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                    name: _('user states'),
                    _id: actionSelectorLvl1OptionIds.USER_STATES,
                    next: true,
                    icon: null,
                    passive: Ext.data.StoreManager.lookup('actionConfigurator_userStateStore').count() === 0
                }]);
            }

            const sonosVdcs = dss.staticDataModel.MeterCache && dss.staticDataModel.MeterCache.filter(function (meter) {
                return meter.ImplementationId === 'Sonos_Container';
            });
            const sonosSpeakers = sonosVdcs && sonosVdcs[0] && dss.staticDataModel.deviceCache.filter(function (device) {
                return device.DSMeterDSUID === sonosVdcs[0].dSUID;
            });
            if (sonosSpeakers && sonosSpeakers.length) {
                Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                    name: _('audio activities'),
                    _id: actionSelectorLvl1OptionIds.AUDIO,
                    next: true,
                    icon: null,
                    passive: Ext.data.StoreManager.lookup('actionConfigurator_audioActivityStore').count() === 0
                }]);
            }

            Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([{
                name: _('call URL'),
                _id: actionSelectorLvl1OptionIds.CALL_URL,
                next: true,
                icon: null
            }, {
                name: '-',
                _id: actionSelectorConsts.SEPARATOR_GRID_ITEM_ID,
                next: false,
                icon: null,
                passive: true
            }, {
                name: _('access action'),
                _id: actionSelectorLvl1OptionIds.ACCESS,
                next: true,
                icon: null
            }, {
                name: _('weather action'),
                _id: actionSelectorLvl1OptionIds.WEATHER,
                next: true,
                icon: null
            }, {
                name: _('security action'),
                _id: actionSelectorLvl1OptionIds.SECURITY,
                next: true,
                icon: null
            }]);
            // if (dss.staticDataModel.ClimateControllerSettingsCache) {
            //     if (dss.staticDataModel.ClimateControllerSettingsCache.seasonSettings!=null)
            //         if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities)
            //             if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.coolingSupported!=null)
            //                 if (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.heatingSupported!=null)
            //                     if ((dss.staticDataModel.ClimateControllerSettingsCache.capabilities.coolingSupported)||
            //                         (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.heatingSupported)) {
            //                             Ext.data.StoreManager.lookup('actionConfigurator_l1Store').add([ {
            //                                 name: _('heating system'),
            //                                 id: actionSelectorLvl1OptionIds.HEATING,
            //                                 next: true,
            //                                 icon: null,
            //                                 passive:false
            //                             } ]);
            //                         }
            //
            // }

        }
        if (Ext.data.StoreManager.lookup('actionConfigurator_zoneStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_zoneStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('actionConfigurator_zoneStore').add(dss.staticDataModel.getZoneArray());
        }
        if (Ext.data.StoreManager.lookup('actionConfigurator_zoneDeviceStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_zoneDeviceStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var zoneArray = dss.staticDataModel.getZoneArray();
            if (zoneArray) {
                for (i = 0; i < zoneArray.length; i++) {
                    var oStore = this.getDeviceStore(dss.staticDataModel.getZoneArray()[i].ZoneID);
                    if (oStore.count() > 0) {
                        Ext.data.StoreManager.lookup('actionConfigurator_zoneDeviceStore').add(dss.staticDataModel.getZoneArray()[i]);
                    }
                }
            }
        }
    },
    initGridStore: function () {
        var me = this;
        this.gridStore = Ext.create('Ext.data.Store', {

            fields: [ 'pos', 'action', 'delay', 'delayHr', 'delayMin', 'delaySec'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            },
            listeners: {
                datachanged: function (store) {     // , eOpts
                    var ct = store.getCount();
                    me.lblSelectorAnzeige.setText(Ext.String.format2(_('{count} activity', '{count} activites', ct), {
                        count: ct.toString()
                    }), false);
                    if (ct == 0) {
                        me.macPanel.setTitle(_('show activity editor'));
                    }
                }
            }
        });
    },
    getGroupStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('actionConfigurator_groupStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_groupStore_' + ZoneID,
                fields: [ 'group', 'name', 'icon', 'reachable' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oGroup = dss.staticDataModel.getGroupForZone(ZoneID);
            Ext.data.StoreManager.lookup('actionConfigurator_groupStore_' + ZoneID).add(oGroup);
            if (this.fHeatingOperationMode) {
                if (dss.staticDataModel.heating[ZoneID].receiveOperationMode) {
                    Ext.data.StoreManager.lookup('actionConfigurator_groupStore_' + ZoneID).add({
                        group: DSS.Groups.TEMPERATURE,
                        name: _('Heating Modes'),
                        icon: dss.staticDataModel.getImagePathForGroup(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE),
                        reachable: true
                    });
                }
            }
        }
        return Ext.data.StoreManager.lookup('actionConfigurator_groupStore_' + ZoneID);
    },
    getSceneStore: function (ZoneID, groupID) {
        if (Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_szeneStore_' + ZoneID + '_' + groupID,
                fields: [ 'id', 'name', 'reachable', 'icon', 'outputMode', 'passive' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            if (groupID == DSS.Groups.TEMPERATURE) {
                if (ZoneID == DSS.Zones.ALL_DEVICES) {
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 4,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('NIGHT Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 5,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('HOLIDAY Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(
                        new ActionGridSeparator({ reachable: true })
                    );
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 4 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset NIGHT Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 5 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset HOLIDAY Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(
                        new ActionGridSeparator({ reachable: false })
                    );
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 2,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.USER1, false),
                        name: _('ECO Mode preferred')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 1,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.USER1, false),
                        name: _('COMFORT Mode preferred')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 3,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('NotUsed Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(
                        new ActionGridSeparator({ reachable: false })
                    );
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 3 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset NotUsed Mode')
                    });
                } else {
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 2,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.USER1, false),
                        name: _('ECO Mode preferred')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 1,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.USER1, false),
                        name: _('COMFORT Mode preferred')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 3,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('NotUsed Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 3 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: true,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset NotUsed Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 4,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('NIGHT Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 4 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset NIGHT Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 5,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('HOLIDAY Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 5 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: false,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset HOLIDAY Mode')
                    });
                    var climateSettings = dss.staticDataModel.getClimateSettings();
                    var showCooling = Boolean(climateSettings && climateSettings.capabilities && climateSettings.capabilities.coolingSupported);
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 6,
                        reachable: showCooling,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('COOLING Mode')
                    });
                    Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                        id: 6 + actionSelectorConsts.RESET_MODE_SHIFT,
                        reachable: showCooling,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, DSS.Groups.CLIMATE, DSS.Scenes.OFF, false),
                        name: _('reset COOLING Mode')
                    });
                }
            } else {
                var oScenes = dss.staticDataModel.getScenes(ZoneID, groupID);
                if (oScenes) {
                    for (var i = 0; i < oScenes.length; i++) {
                        var oObject = {};
                        oObject.id = oScenes[i].id;
                        oObject.reachable = oScenes[i].reachable;
                        oObject.icon = oScenes[i].icon;
                        if (oScenes[i].name == oScenes[i].defaultName) {
                            oObject.name = oScenes[i].name;
                        } else {
                            oObject.name = oScenes[i].defaultName + ' - ' + oScenes[i].name;
                        }

                        if (ZoneID == DSS.Zones.ALL_DEVICES) {
                            if (groupID == DSS.Groups.ALL) {
                                if (oScenes[i].id == DSS.Scenes.HAIL) {
                                    oObject.reachable = false;
                                }
                                if (oScenes[i].id == DSS.Scenes.NO_HAIL) {
                                    oObject.reachable = false;
                                }
                            }
                        }

                        Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(oObject);

                        if (oObject.undoable) {
                            oObject = {};
                            oObject.id = -1 * oScenes[i].id;
                            oObject.reachable = oScenes[i].reachable;
                            oObject.icon = oScenes[i].icon;


                            if (oScenes[i].name == oScenes[i].defaultName) {
                                oObject.name = _('Undo ') + oScenes[i].name;
                            } else {
                                oObject.name = _('Undo ') + oScenes[i].defaultName + ' - ' + oScenes[i].name;
                            }
                            Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(oObject);
                        }
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('actionConfigurator_szeneStore_' + ZoneID + '_' + groupID);
    },
    getDeviceStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('actionConfigurator_deviceStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'actionConfigurator_deviceStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present', 'outputMode', 'specialActions' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDevice = dss.staticDataModel.getDevicesForZone(ZoneID, 0);
            if (oDevice) {
                for (var i = 0; i < oDevice.length; i++) {
                    // var fShouldDo = oDevice[i].outputMode != 0;
                    var fShouldDo = oDevice[i].outputMode != 0 && oDevice[i].outputMode != 30 && oDevice[i].outputMode != 64;

                    if (oDevice[i].deviceCustomActions) {
                        if (Object.keys(oDevice[i].deviceCustomActions).length > 0) {
                            fShouldDo = true;
                        }
                    }
                    if (oDevice[i].deviceStandardActions) {
                        if (Object.keys(oDevice[i].deviceStandardActions).length > 0) {
                            fShouldDo = true;
                        }
                    }
                    if (oDevice[i].specialActions.actions) {
                        if (oDevice[i].specialActions.actions.deviceActions != null) {
                            fShouldDo = true;
                        }
                    }

                    if (!oDevice[i].present) {
                        if (!this.fIgnorePresence) {
                            fShouldDo = false;
                        }
                    }
                    if (fShouldDo) {
                        Ext.data.StoreManager.lookup('actionConfigurator_deviceStore_' + ZoneID).add(oDevice[i]);
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('actionConfigurator_deviceStore_' + ZoneID);
    },
    constructGridLvlUserGroup: function () {
        var oGroupControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'group', 'name', 'icon', 'reachable' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oGroups = dss.staticDataModel.getUsergroups();
        if (oGroups) {
            for (var i = 0; i < oGroups.length; i++) {
                oGroupControlStore.add({
                    id: oGroups[i].id,
                    group: oGroups[i].id,
                    name: oGroups[i].name,
                    icon: dss.staticDataModel.getImagePathForGroup(DSS.Zones.ALL_DEVICES, oGroups[i].id),
                    reachable: true
                });
                if (oGroups[i].color == 1) {
                    oGroupControlStore.add({
                        id: -1 * oGroups[i].id,
                        group: null,
                        name: oGroups[i].name + ' ' + _('blinking'),
                        icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png',
                        reachable: true
                    });
                }
            }
        }

        this.grids.lvlUserGroup = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            hideHeaders: true,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oGroupControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlUserGroup, this.actionSelectorName + 'GridLvlUserGroup');
        this.grids.lvlUserGroup.on('itemdblclick', function () {    // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.grids.lvlUserGroup.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlUserGroup.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlUsergroupScene && this.parent.macPanel.remove(this.parent.grids.lvlUsergroupScene);
            var id = record.get('id');
            if (id != null) {
                if (id < 0) {
                    var oObject = {
                        type: DSS.ActionTypes.ZONE_BLINK,
                        params: {
                            zone: DSS.Zones.ALL_DEVICES,
                            group: Math.abs(id)
                        }
                    };
                    this.parent.setSelectedSubItem(oObject, true);
                } else {
                    this.parent.constructGridLvlUserGroupScene(id);
                }
            }
        });


        this.grids.lvlUserGroup.parent = this;
        this.macPanel.add(this.grids.lvlUserGroup);
    },
    constructGridLvlUserGroupScene: function (groupID, fAll, fForce) {
        var oSceneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'name', 'icon', 'reachable', 'checkable', 'checked', 'passive' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (fAll == null) {
            fAll = false;
        }
        if (fForce == null) {
            fForce = false;
        }
        var oSceneStore = this.getSceneStore(0, groupID);
        if (groupID == DSS.Groups.TEMPERATURE && !this.fHeatingExtendedOperationMode) {

        } else {
            oSceneControlStore.add(
                new ActionGridShowAllCheckbox({ checked: fAll })
            );
            oSceneControlStore.add(
                new ActionGridSeparator({ reachable: true })
            );
        }
        fillAndAmendSceneStoreWithAllOff({
            sceneStore: oSceneStore,
            storeToFillAndAmend: oSceneControlStore,
            allScenesVisible: fAll,
            sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
            excludedGroups: [ DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],
            groupID: groupID
        });

        var localWidth = 474;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 2 * this.iDefaultWidth - 2;
        }

        this.grids.lvlUsergroupScene = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            hideHeaders: true,
            viewConfig: {
                stripeRows: true
            },
            store: oSceneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlUsergroupScene, this.actionSelectorName + 'GridLvlUserGroupScene');
        this.grids.lvlUsergroupScene.parent = this;
        this.grids.lvlUsergroupScene.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlUsergroupScene.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var groupID = this.parent.grids.lvlUserGroup.getSelected('id');
            if (record.get('checkable')) {
                var fAll = !record.get('checked');
                record.set('checked', fAll);
                grid.getSelectionModel().deselectAll();
                if (record.get('id') == actionSelectorConsts.SHOW_ALL_GRID_ITEMS_ID) {
                    // alle Stimmungen
                    grid.store.removeAll();

                    grid.store.add(
                        new ActionGridShowAllCheckbox({ checked: record.get('checked') })
                    );
                    grid.store.add(
                        new ActionGridSeparator({ reachable: true })
                    );
                    fillAndAmendSceneStoreWithAllOff({
                        sceneStore: oSceneStore,
                        storeToFillAndAmend: grid.store,
                        allScenesVisible: fAll,
                        sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
                        excludedGroups: [ DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],
                        groupID: groupID
                    });
                }
            }
            if (record.get('id') >= 0) {
                if (groupID != null) {
                    var fForce = false;
                    if (record.get('force') != null) {
                        fForce = record.get('force');
                    }
                    var oObject = {
                        type: DSS.ActionTypes.ZONE_SCENE,
                        params: {
                            zone: DSS.Zones.ALL_DEVICES,
                            group: groupID,
                            scene: record.get('id'),
                            force: fForce
                        }
                    };
                    this.parent.setSelectedSubItem(oObject, true);
                }
            }
        });
        this.grids.lvlUsergroupScene.on('itemdblclick', function () {   // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlUsergroupScene);

    },
    constructSimpleSceneGrid: function (storeName) {        // template for access, security and weather
        var localWidth = 624;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 1 * this.iDefaultWidth - 2;
        }
        const simpleGrid = Ext.create('DSS.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: Ext.data.StoreManager.lookup(storeName),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('scene'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });

        simpleGrid.parent = this;
        simpleGrid.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        simpleGrid.on('itemdblclick', function () {     // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        return simpleGrid;
    },
    constructGridLvlAccess: function () {
        this.grids.lvlAccess = this.constructSimpleSceneGrid('actionConfigurator_accessStore');
        addDataIdToElement(this.grids.lvlAccess, this.actionSelectorName + 'GridLvlAccess');

        this.grids.lvlAccess.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;

            var oObject;
            if (record.get('id') < actionSelectorConsts.UNDO_SCENE_SHIFT) {
                oObject = {
                    type: DSS.ActionTypes.ZONE_SCENE,
                    params: {
                        zone: DSS.Zones.ALL_DEVICES,
                        group: DSS.Groups.ALL,
                        scene: record.get('id'),
                        force: false
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            } else {
                oObject = {
                    type: DSS.ActionTypes.UNDO_ZONE_SCENE,
                    params: {
                        zone: DSS.Zones.ALL_DEVICES,
                        group: DSS.Groups.ALL,
                        scene: record.get('id') - actionSelectorConsts.UNDO_SCENE_SHIFT,
                        force: false
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            }
        });

        this.macPanel.add(this.grids.lvlAccess);
    },
    // constructGridLvlHeating: function () {
    //
    //     var localWidth=624;
    //     if (this.fUseAcutalWidth) {
    //         localWidth=this.getWidth()-(1 * this.iDefaultWidth)-2;
    //     }
    //
    //     var me=this;
    //     this.grids.lvlHeating = Ext.create('DSS.grid.Panel', {
    //         hideHeaders:true,
    //         height: this.iDefaultHeight,
    //         width: localWidth,
    //         autoScroll: true,
    //         viewConfig : {
    //             stripeRows: true
    //         },
    //         store: Ext.data.StoreManager.lookup('actionConfigurator_heatingSystemStore'),
    //         columns : [ {
    //             sortable: false,
    //             hideable: false,
    //             text: _('scene'),
    //             dataIndex: 'name',
    //             flex: 1,
    //             renderer: this.defMacGridRenderer
    //         } ]
    //     });
    //     addDataIdToElement(this.grids.lvlHeating, this.actionSelectorName + 'GridLvlHeating');
    //
    //     this.grids.lvlHeating.parent = this;
    //             this.grids.lvlHeating.on('beforeitemclick', function (grid, record) {
    //     return !record.get('passive');
    // });
    //
    //     this.grids.lvlHeating.on('itemclick', function (grid, record) {
    //                 if (record.get('passive')) {
    //     return;
    // }
    //         this.parent.oActualValue=null;
    //         var oObject;
    //
    //         if (record.get('id')=="heating_mode.off") {
    //             oObject = {
    //                 type : DSS.ActionTypes.CHANGE_ADDON_STATE,
    //                 params: {
    //                     statename : 'apartment_controller',
    //                     'addonId': 'system-addon-heating-controller',
    //                     state: 'off',
    //                     value: 0
    //                 }
    //             };
    //             this.parent.setSelectedSubItem(oObject,true);
    //         }
    //         if (record.get('id')=="heating_mode.heating") {
    //             oObject = {
    //                 type : DSS.ActionTypes.CHANGE_ADDON_STATE,
    //                 params: {
    //                     statename : 'apartment_controller',
    //                     'addonId': 'system-addon-heating-controller',
    //                     state: 'heating',
    //                     value: 1
    //                 }
    //             };
    //             this.parent.setSelectedSubItem(oObject,true);
    //         }
    //         if (record.get('id')=="heating_mode.cooling") {
    //             oObject = {
    //                 type : DSS.ActionTypes.CHANGE_ADDON_STATE,
    //                 params: {
    //                     statename : 'apartment_controller',
    //                     'addonId': 'system-addon-heating-controller',
    //                     state: 'cooling',
    //                     value: 2
    //                 }
    //             };
    //             this.parent.setSelectedSubItem(oObject,true);
    //         }
    //     });
    //     this.grids.lvlHeating.on('itemdblclick', function (grid, record) {
    //         if (this.parent.oActualValue !=null) {
    //             this.parent.macPanel.collapse();
    //         }
    //     });
    //
    //     this.macPanel.add(this.grids.lvlHeating);
    //
    // },
    constructGridLvlUserStateValue: function (stateID) {

        var localWidth = 324;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 3 * this.iDefaultWidth - 2;
        }


        var oAddonStateStore = Ext.create('Ext.data.Store', {
            fields: [ 'value', 'name', 'title', 'passive', 'icon' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oState = null;
        if (dss.staticDataModel.userStates) {
            for (var i = 0; i < dss.staticDataModel.userStates.length; i++) {
                if (dss.staticDataModel.userStates[i].name == stateID) {
                    oState = dss.staticDataModel.userStates[i];
                }
            }
        }
        if (oState) {
            var oItem = {
                name: stateID,
                value: 'active',
                title: oState.setName,
                icon: dss.staticDataModel.baseImagePath + 'state/uds_state_active.png',
                passive: false
            };
            oAddonStateStore.add(oItem);
            oItem = {
                name: stateID,
                value: 'inactive',
                title: oState.resetName,
                icon: dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png',
                passive: false
            };
            oAddonStateStore.add(oItem);
        }
        this.grids.lvlUserStateValue = Ext.create('DSS.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oAddonStateStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'title',
                flex: 1,
                renderer: function (val, metadata, record) {        // , colIdx, store, view
                    return '<table style="height:16px;width:100%;" data-name="' + record.get('name') + '" data-value="' + record.get('value') + '">' +
                        '<tr style="height:16px;">' +
                        '    <td><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>' +
                        '    <td width="100%">' + record.get('title') + '</td>' +
                        '</tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlUserStateValue, this.actionSelectorName + 'GridLvlUserDefinedStateValue');

        this.grids.lvlUserStateValue.parent = this;
        this.grids.lvlUserStateValue.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlUserStateValue.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;

            var oObject = {
                type: DSS.ActionTypes.CHANGE_ADDON_STATE,
                params: {
                    statename: record.get('name'),
                    addonId: 'system-addon-user-defined-states',
                    state: record.get('value')
                }
            };
            this.parent.setSelectedSubItem(oObject, true);

        });

        this.grids.lvlUserStateValue.on('itemdblclick', function () {   // grid, record
            if (this.parent.lvlUserStateValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlUserStateValue);

    },
    constructGridLvlUserState: function () {
        this.grids.lvlUserState = Ext.create('DSS.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 300,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: Ext.data.StoreManager.lookup('actionConfigurator_userStateStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('userState'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlUserState, this.actionSelectorName + 'GridLvlUserDefinedState');

        this.grids.lvlUserState.parent = this;
        this.grids.lvlUserState.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlUserState.on('itemclick', function (grid, record) {
            this.parent.grids.lvlUserStateValue && this.parent.macPanel.remove(this.parent.grids.lvlUserStateValue);
            this.parent.constructGridLvlUserStateValue(record.get('_id'));
        });
        this.macPanel.add(this.grids.lvlUserState);
    },
    constructGridLvlAudio: function () {
        let localWidth = 624;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 1 * this.iDefaultWidth - 2;
        }
        // this.grids.lvlAudio = Ext.create('DSS.grid.Panel', {
        this.grids.lvlAudio = Ext.create('DSS.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: Ext.data.StoreManager.lookup('actionConfigurator_audioActivityStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('audio activity'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlAudio, this.actionSelectorName + 'GridLvlAudio');

        this.grids.lvlAudio.parent = this;
        this.grids.lvlAudio.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlAudio.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var oObject = {
                type: DSS.ActionTypes.AUDIO_ACTIVITY,
                params: {
                    dsuid: record.get('vdcDsuid'),
                    powerState: 1,
                    contentSource: record.get('id'),
                    volume: record.get('volume'),
                    repeatMode: record.get('repeat') ? audioActionRepeatMode.INDEFINITELY : audioActionRepeatMode.NEVER,
                    shuffleMode: Boolean(record.get('shuffle')),
                    zones: '[' + record.get('zones').map(function (zoneId) { return parseInt(zoneId); }).join(',') + ']'
                }
            };
            this.parent.setSelectedSubItem(oObject, true);
        });
        this.macPanel.add(this.grids.lvlAudio);
    },
    constructGridLvlWeather: function () {
        this.grids.lvlWeather = this.constructSimpleSceneGrid('actionConfigurator_weatherStore');
        addDataIdToElement(this.grids.lvlWeather, this.actionSelectorName + 'GridLvlWeather');

        this.grids.lvlWeather.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;

            var sValue = record.get('id');
            var stateName = sValue.substr(0, sValue.lastIndexOf('_'));
            var stateValue = sValue.substr(sValue.lastIndexOf('_') + 1);
            var oObject = {
                type: DSS.ActionTypes.CHANGE_STATE,
                params: {
                    statename: stateName,
                    state: stateValue
                }
            };
            this.parent.setSelectedSubItem(oObject, true);
        });

        this.macPanel.add(this.grids.lvlWeather);
    },
    constructGridLvlPanic: function () {
        this.grids.lvlPanic = this.constructSimpleSceneGrid('actionConfigurator_securityStore');
        addDataIdToElement(this.grids.lvlPanic, this.actionSelectorName + 'GridLvlSecurity');

        this.grids.lvlPanic.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var oObject;

            if (record.get('id').toString().indexOf('_') >= 0) {
                var sValue = record.get('id');
                var stateName = sValue.substr(0, sValue.lastIndexOf('_'));
                var stateValue = sValue.substr(sValue.lastIndexOf('_') + 1);
                oObject = {
                    type: DSS.ActionTypes.CHANGE_STATE,
                    params: {
                        statename: stateName,
                        state: stateValue
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            } else if (record.get('id') < actionSelectorConsts.UNDO_SCENE_SHIFT) {
                oObject = {
                    type: DSS.ActionTypes.ZONE_SCENE,
                    params: {
                        zone: DSS.Zones.ALL_DEVICES,
                        group: DSS.Groups.ALL,
                        scene: record.get('id'),
                        force: false
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            } else {
                oObject = {
                    type: DSS.ActionTypes.UNDO_ZONE_SCENE,
                    params: {
                        zone: DSS.Zones.ALL_DEVICES,
                        group: DSS.Groups.ALL,
                        scene: record.get('id') - actionSelectorConsts.UNDO_SCENE_SHIFT,
                        force: false
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            }
        });

        this.macPanel.add(this.grids.lvlPanic);
    },
    constructGridLvlDeviceScene: function (zoneID, groupID, fAll, fForce) {
        var oSceneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'name', 'icon', 'reachable', 'checkable', 'checked', 'passive' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (fAll == null) {
            fAll = false;
        }
        if (fForce == null) {
            fForce = false;
        }

        var calcWidth = this.calculateWidthHelper('x-grid-cell');
        var oSceneStore = this.getSceneStore(zoneID, groupID);
        oSceneControlStore.add(
            new ActionGridShowAllCheckbox({ checked: fAll })
        );
        oSceneControlStore.add(
            new ActionGridSeparator({ reachable: true })
        );
        var maxLabelWidth = fillAndAmendSceneStoreWithAllOff({
            sceneStore: oSceneStore,
            storeToFillAndAmend: oSceneControlStore,
            allScenesVisible: fAll,
            sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
            initialMaxLabelWidth: calcWidth(_('show all scenes')),
            calcWidth: calcWidth,
            excludedGroups: [ DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],
            groupID: groupID
        });

        var localWidth = 174;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 4 * this.iDefaultWidth - 2;
        }

        this.grids.lvlDeviceScene = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: Math.max(localWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            hideHeaders: true,
            store: oSceneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceScene, this.actionSelectorName + 'GridLvlSingleDeviceZoneDeviceActionScene');
        this.grids.lvlDeviceScene.parent = this;
        this.macPanel.add(this.grids.lvlDeviceScene);

        this.grids.lvlDeviceScene.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlDeviceScene.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            if (record.get('checkable')) {
                var fAll = !record.get('checked');
                record.set('checked', fAll);
                grid.getSelectionModel().deselectAll();
                if (record.get('id') == actionSelectorConsts.SHOW_ALL_GRID_ITEMS_ID) {
                    // alle Stimmungen
                    grid.store.removeAll();

                    grid.store.add(
                        new ActionGridShowAllCheckbox({ checked: record.get('checked') })
                    );
                    grid.store.add(
                        new ActionGridSeparator({ reachable: true })
                    );
                    fillAndAmendSceneStoreWithAllOff({
                        sceneStore: oSceneStore,
                        storeToFillAndAmend: grid.store,
                        allScenesVisible: fAll,
                        sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
                        excludedGroups: [ DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],
                        groupID: groupID
                    });
                }
                return;
            }
            if (this.parent.grids.lvl1.getSelected('_id') == 1) {
                var dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                if (dSUID != null) {
                    var fForce = false;
                    if (record.get('force') != null) {
                        fForce = record.get('force');
                    }
                    var oObject = {
                        type: DSS.ActionTypes.DEVICE_SCENE,
                        params: {
                            dsuid: dSUID,
                            scene: record.get('id'),
                            force: fForce
                        }
                    };
                    this.parent.setSelectedSubItem(oObject, true);
                }
            }
        });
        this.grids.lvlDeviceScene.on('itemdblclick', function () {  // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

    },
    constructGridLvlDeviceCategoryAction: function (dSUID, categoryName) {
        var actionStandard = dss.staticDataModel.getStandardActions(dSUID);
        var oDeviceActions = dss.staticDataModel.getDevice(dSUID).deviceActions;
        var localWidth = this.getWidth() - 4 * this.iDefaultWidth - 2;

        var oDeviceActionControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'name', 'icon', 'next' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        for (var sKey in actionStandard) {
            if (actionStandard[sKey].description.category != null) {
                if (actionStandard[sKey].description.category == categoryName) {
                    oDeviceActionControlStore.add({
                        id: 'actions_' + sKey,
                        name: actionStandard[sKey].title,
                        icon: null,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(actionStandard[sKey].title));
                }
            }
        }

        if (oDeviceActions) {
            for (var sKey in oDeviceActions) {
                if (oDeviceActions[sKey].category != null) {
                    if (oDeviceActions[sKey].category == categoryName) {
                        oDeviceActionControlStore.add({
                            id: 'actions_' + sKey,
                            name: oDeviceActions[sKey].title,
                            icon: null,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                    }
                }
            }
        }


        this.grids.lvlDeviceActionSub = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: Math.max(localWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            hideHeaders: true,
            store: oDeviceActionControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceActionSub, this.actionSelectorName + 'GridLvlSingleDeviceZoneDeviceActionSub');
        this.grids.lvlDeviceActionSub.parent = this;
        this.grids.lvlDeviceActionSub.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlDeviceActionSub.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var dSUID;
            var oObject;
            var id = record.get('id');

            if (id.toString().indexOf('actions_') === 0) {
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                var actionName = id.substr(8);
                oObject = {
                    type: DSS.ActionTypes.DEVICE_ACTION,
                    params: {
                        dsuid: dSUID,
                        id: actionName
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            }
        });

        this.grids.lvlDeviceActionSub.on('itemdblclick', function () {  // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlDeviceActionSub);

    },
    constructGridLvlDeviceActions: function (dSUID) {
        this.selectedColor = null;
        this.selectedColTemperature = null;
        var sKey;
        var iGroup;
        var oDeviceActionControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'name', 'icon', 'next' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDeviceObject = dss.staticDataModel.getDevice(dSUID);

        var localWidth = 324;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 3 * this.iDefaultWidth - 2;
        }

        const fShouldShowImpulse = dss.staticDataModel.deviceCanDoImpulse(oDeviceObject);
        var fSubelements = true;

        var fSpecialAction = dss.staticDataModel.deviceHasSpecialActions(oDeviceObject);
        if (fSpecialAction) {
            iGroup = oDeviceObject.GroupID;
            if (iGroup == DSS.Groups.JOKER) {
                iGroup = oDeviceObject.buttonActiveGroup;
            }

            for (sKey in oDeviceObject.specialActions.actions.deviceActions) {
                var oActionObject = oDeviceObject.specialActions.actions.deviceActions[sKey];

                var sName = oActionObject.name;

                if (oActionObject.scene != null) {
                    oDeviceActionControlStore.add({
                        id: Number(sKey) + actionSelectorConsts.SPECIAL_ACTION_SHIFT,
                        name: sName,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, Number(oActionObject.scene)),
                        next: false
                    });
                } else {
                    oDeviceActionControlStore.add({
                        id: Number(sKey) + actionSelectorConsts.SPECIAL_ACTION_SHIFT,
                        name: sName,
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, (Number(oActionObject.outputValue) > 0 ? 17 : 0)),
                        next: false
                    });
                }
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(sName));
            }
            fSubelements = false;
        }
        if (oDeviceObject.hasActions) {
            fSubelements = false;

            if (fSpecialAction) {
                oDeviceActionControlStore.add({
                    id: -2,
                    name: '-',
                    icon: null,
                    next: false
                });
            }

            var oDevice = dss.staticDataModel.getDevice(dSUID);
            var oCustom = oDevice.deviceCustomActions;
            var oStandard = oDevice.deviceStandardActions;
            var oDeviceActions = oDevice.deviceActions;
            var oCategory = oDevice.deviceCategory;
            var oDescriptions = oDevice.deviceActionDescription;
            var oActionArray = [];
            var oStandardActionArray = [];
            var oCustomActionArray = [];

            var oActionCatsAndDogs = {};

            if (oStandard) {
                for (var sKey in oStandard) {
                    var description = oDescriptions[oStandard[sKey].action];
                    var selectedCategory = null;
                    if (description && description.category && oCategory) {
                        selectedCategory = oCategory[description.category];
                    }
                    if (selectedCategory == null) {
                        oStandardActionArray.push({
                            id: 'actions_' + sKey,
                            name: oStandard[sKey].title,
                            icon: null,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStandard[sKey].title));
                    } else {
                        if (oActionCatsAndDogs[description.category] == null) {
                            oActionCatsAndDogs[description.category] = [];
                        }
                        if (!selectedCategory.alwaysVisible) {
                            fSubelements = true;
                            oActionCatsAndDogs[description.category] = [ {
                                id: 'category_' + description.category,
                                name: selectedCategory.title,
                                icon: null,
                                next: true
                            } ];
                        } else {
                            oActionCatsAndDogs[description.category].push({
                                id: 'actions_' + sKey,
                                name: oStandard[sKey].title,
                                icon: null,
                                next: false
                            });
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStandard[sKey].title));
                        }
                    }
                }
            }

            if (oCustom) {
                for (var sKey in oCustom) {
                    if (oActionCatsAndDogs.__custom == null) {
                        oActionCatsAndDogs.__custom = [];
                    }
                    oActionCatsAndDogs.__custom.push({
                        id: 'actions_' + sKey,
                        name: oCustom[sKey].title,
                        icon: null,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oCustom[sKey].title));
                }
            }

            if (oDeviceActions) {
                for (var sKey in oDeviceActions) {
                    var selectedCategory = null;
                    if (oDeviceActions[sKey].category && oCategory) {
                        selectedCategory = oCategory[oDeviceActions[sKey].category];
                    }
                    if (selectedCategory == null) {
                        oActionArray.push({
                            id: 'actions_' + sKey,
                            name: oDeviceActions[sKey].title,
                            icon: null,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                    } else {
                        if (oActionCatsAndDogs[oDeviceActions[sKey].category] == null) {
                            oActionCatsAndDogs[oDeviceActions[sKey].category] = [];
                        }
                        if (!selectedCategory.alwaysVisible) {
                            fSubelements = true;
                            oActionCatsAndDogs[oDeviceActions[sKey].category] = [ {
                                id: 'category_' + oDeviceActions[sKey].category,
                                name: selectedCategory.title,
                                icon: null,
                                next: true
                            } ];
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(selectedCategory.title));
                        } else {
                            oActionCatsAndDogs[oDeviceActions[sKey].category].push({
                                id: 'actions_' + sKey,
                                name: oDeviceActions[sKey].title,
                                icon: null,
                                next: false
                            });
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                        }
                    }
                }
            }
            var orderArray = [];
            for (var sKey in oActionCatsAndDogs) { orderArray.push(sKey); }

            orderArray.sort(function (aKey, bKey) {
                if (aKey === '__custom') {
                    return 1;
                }
                if (bKey === '__custom') {
                    return -1;
                }
                var a = oCategory[aKey];
                var b = oCategory[bKey];
                return Number(a.orderId) < Number(b.orderId)
                    ? -1
                    : 1;
            });

            for (var i = 0; i < orderArray.length; i++) {
                oActionCatsAndDogs[orderArray[i]].sort(function (a, b) {
                    return a.name.toUpperCase() < b.name.toUpperCase()
                        ? -1
                        : 1;
                });

                oDeviceActionControlStore.add(oActionCatsAndDogs[orderArray[i]]);

                if (i < orderArray.length - 1) {
                    var fSkip = false;
                    if (oActionCatsAndDogs[orderArray[i]].length === 1) {
                        if (oActionCatsAndDogs[orderArray[i]][0].id.indexOf('category') === 0) {
                            if (oActionCatsAndDogs[orderArray[i + 1]].length === 1) {
                                if (oActionCatsAndDogs[orderArray[i + 1]][0].id.indexOf('category') === 0) {
                                    fSkip = true;
                                }
                            }
                        }
                    }
                    if (!fSkip) {
                        oDeviceActionControlStore.add({
                            id: -1 * (4 + i),
                            name: '-',
                            icon: null,
                            next: false
                        });
                    }
                }
            }

            if (oDeviceActionControlStore.count() > 0 && oStandardActionArray.length + oCustomActionArray.length + oActionArray.length > 0) {
                oDeviceActionControlStore.add({
                    id: -1 * (oDeviceActionControlStore.count()),
                    name: '-',
                    icon: null,
                    next: false
                });
            }
            oDeviceActionControlStore.add(oStandardActionArray);
            oDeviceActionControlStore.add(oCustomActionArray);
            oDeviceActionControlStore.add(oActionArray);
        }

        if (!oDeviceObject.hasActions && !fSpecialAction) {
            var sIconOff;
            var sIconOn;
            var sIcon;
            if (dss.staticDataModel.deviceHasTwoStages(oDeviceObject)) {
                /* UMR special Mode*/
                iGroup = oDeviceObject.GroupID;
                if (iGroup == DSS.Groups.JOKER) {
                    iGroup = oDeviceObject.buttonActiveGroup;
                }
                if (this.fUMRExtendetFunctions || oDeviceObject.GroupID != DSS.Groups.JOKER) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.CALL_SCENE,
                        name: _('call scene'),
                        icon: null,
                        next: true
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('call scene')));
                }

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_off.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.OFF);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_0_OF_3,
                    name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off')));

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_33.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER1);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_1_OF_3,
                    name: _('Stage 1'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Stage 1')));

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_66.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER2);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_2_OF_3,
                    name: _('Stage 2'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Stage 2')));

                if (oDeviceObject.GroupID == DSS.Groups.LIGHT) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.BLINK,
                        name: _('Blink'),
                        icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png',
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Blink')));
                }

            } else if (dss.staticDataModel.deviceHasThreeStages(oDeviceObject)) {
                /* UMR special Mode*/
                iGroup = oDeviceObject.GroupID;
                if (iGroup == DSS.Groups.JOKER) {
                    iGroup = oDeviceObject.buttonActiveGroup;
                }
                if (this.fUMRExtendetFunctions || oDeviceObject.GroupID != DSS.Groups.JOKER) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.CALL_SCENE,
                        name: _('call scene'),
                        icon: null,
                        next: true
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('call scene')));
                }

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_off.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.OFF);
                }

                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_0_OF_4,
                    name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off')));

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_25.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER1);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_1_OF_4,
                    name: _('Stage 1'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Stage 1')));

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_50.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER2);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_2_OF_4,
                    name: _('Stage 2'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Stage 2')));

                if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                    sIcon = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_75.png';
                } else {
                    sIcon = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER3);
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.LEVEL_3_OF_4,
                    name: _('Stage 3'),
                    icon: sIcon,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Stage 3')));

                if (oDeviceObject.GroupID == DSS.Groups.LIGHT) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.BLINK,
                        name: _('Blink'),
                        icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png',
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Blink')));
                }

            } else if (oDeviceObject.GroupID == DSS.Groups.LIGHT || oDeviceObject.buttonActiveGroup != DSS.Groups.JOKER) {
                iGroup = oDeviceObject.GroupID;
                if (iGroup == DSS.Groups.JOKER) {
                    iGroup = oDeviceObject.buttonActiveGroup;
                }

                if (oDeviceObject.functionID != 28688) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.CALL_SCENE,
                        name: _('call scene'),
                        icon: null,
                        next: true
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('call scene')));
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.OFF,
                        name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off'),
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.OFF),
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off')));
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.ON,
                        name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Open') : _('Turn on'),
                        icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, iGroup, DSS.Scenes.USER1),
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Open') : _('Turn on')));
                }

                if (!fShouldShowImpulse) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.BLINK,
                        name: _('Blink'),
                        icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png',
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Blink')));
                }
            } else if (oDeviceObject.GroupID == DSS.Groups.JOKER) {
                sIconOff = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.OFF);
                sIconOn = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.USER1);
                if (dss.staticDataModel.deviceCanDoImpulse(oDeviceObject)) {
                    sIconOff = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_off.png';
                    sIconOn = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse_on.png';
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.OFF,
                    name: _('Turn off'),
                    icon: sIconOff,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Turn off')));
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.ON,
                    name: _('Turn on'),
                    icon: sIconOn,
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Turn on')));
            } else {
                sIconOff = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.OFF);
                sIconOn = dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.USER1);
                var sIconBlink = dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png';
                if (typeof oDeviceObject.GroupID === 'undefined') {
                    if (oDeviceObject.GroupMembership[0] === DSS.Groups.JOKER) {
                        switch (oDeviceObject.GroupMembership[1]) {
                            case DSS.Groups.APARTMENT_VENTILATION: {
                                sIconOff = dss.staticDataModel.baseImagePath + 'groupIcons/group_64_deact.png';
                                sIconOn = dss.staticDataModel.baseImagePath + 'groupIcons/group_64_active.png';
                                break;
                            }
                        }
                    }
                } else if (oDeviceObject.GroupID === DSS.Groups.RECIRCULATION) {
                    sIconOff = dss.staticDataModel.baseImagePath + 'groupIcons/group_64_deact.png';
                    sIconOn = dss.staticDataModel.baseImagePath + 'groupIcons/group_64_active.png';
                }
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.CALL_SCENE,
                    name: _('call scene'),
                    icon: null,
                    next: true
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('call scene')));
                if (oDeviceObject.GroupID !== DSS.Groups.TEMPERATURE &&
                    (typeof oDeviceObject.GroupID !== 'undefined' || oDeviceObject.GroupMembership[1] !== DSS.Groups.TEMPERATURE)
                ) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.OFF,
                        name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off'),
                        icon: sIconOff,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Close') : _('Turn off')));
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.ON,
                        name: oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Open') : _('Turn on'),
                        icon: sIconOn,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceObject.GroupID == DSS.Groups.SHADOW ? _('Open') : _('Turn on')));
                }
                if (!fShouldShowImpulse) {
                    oDeviceActionControlStore.add({
                        id: actionSelectorDeviceActionOptionIds.BLINK,
                        name: _('Blink'),
                        icon: sIconBlink,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Blink')));
                }
            }
        }
        if (oDeviceObject.moc.length) {
            if (oDeviceObject.moc.indexOf(dss.mocChannelIds.BRIGHTNESS) > -1 &&
                oDeviceObject.moc.indexOf(dss.mocChannelIds.HUE) > -1 &&
                oDeviceObject.moc.indexOf(dss.mocChannelIds.SATURATION) > -1) {
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.COLOR,
                    name: _('Select color'),
                    icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.USER1),
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Select color')));
            }
            if (oDeviceObject.moc.indexOf(dss.mocChannelIds.BRIGHTNESS) > -1 &&
                oDeviceObject.moc.indexOf(dss.mocChannelIds.COLOR_TEMPERATURE) > -1) {
                oDeviceActionControlStore.add({
                    id: actionSelectorDeviceActionOptionIds.COLOR_TEMPERATURE,
                    name: _('Select color temperature'),
                    icon: dss.staticDataModel.getImagePathForScene(DSS.Zones.ALL_DEVICES, oDeviceObject.GroupID, DSS.Scenes.USER1),
                    next: false
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Select color temperature')));
            }
        }
        if (fShouldShowImpulse) {
            oDeviceActionControlStore.add({
                id: actionSelectorDeviceActionOptionIds.IMPULSE,
                name: _('Impulse'),
                icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_impulse.png',
                next: false
            });
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('Impulse')));
        }
        this.grids.lvlDeviceAction = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: Math.max(fSubelements ? this.iDefaultWidth : localWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            hideHeaders: true,
            store: oDeviceActionControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceAction, this.actionSelectorName + 'GridLvlSingleDeviceZoneDeviceAction');
        this.grids.lvlDeviceAction.parent = this;
        this.grids.lvlDeviceAction.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlDeviceAction.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionSub);
            var dSUID;
            var oDeviceObject;
            var oObject;
            var id = record.get('id');
            if (id === actionSelectorDeviceActionOptionIds.COLOR) {
                this.parent.oColorEditor.openColorPickerWindow();
                if (this.parent.selectedColor != null) {
                    this.parent.oColorEditor.setValue(this.parent.selectedColor);
                }
            }
            if (id === actionSelectorDeviceActionOptionIds.COLOR_TEMPERATURE) {
                this.parent.oColTempEditor.show();
                if (this.parent.selectedColTemperature != null) {
                    this.parent.oColTempEditor.setValue(this.parent.selectedColTemperature.temperature, this.parent.selectedColTemperature.bright);
                }
            }

            if (id.toString().indexOf('category_') === 0) {
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                var categoryName = id.substr(9);
                this.parent.constructGridLvlDeviceCategoryAction(dSUID, categoryName);
            } else if (id.toString().indexOf('actions_') === 0) {
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                var actionName = id.substr(8);
                oObject = {
                    type: DSS.ActionTypes.DEVICE_ACTION,
                    params: {
                        dsuid: dSUID,
                        id: actionName
                    }
                };
                this.parent.setSelectedSubItem(oObject, true);
            } else if (id >= actionSelectorConsts.SPECIAL_ACTION_SHIFT) {
                // special actions
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                oDeviceObject = dss.staticDataModel.getDevice(dSUID);
                var sKey = id - actionSelectorConsts.SPECIAL_ACTION_SHIFT;
                var oAction = oDeviceObject.specialActions.actions.deviceActions[sKey];
                if (oAction.scene != null) {
                    oObject = {
                        type: DSS.ActionTypes.DEVICE_SCENE,
                        params: {
                            dsuid: dSUID,
                            scene: oAction.scene,
                            force: false
                        }
                    };
                } else {
                    oObject = {
                        type: DSS.ActionTypes.DEVICE_VALUE,
                        params: {
                            dsuid: dSUID,
                            value: oAction.outputValue
                        }
                    };
                }
                this.parent.setSelectedSubItem(oObject, true);
            } else if (id == actionSelectorDeviceActionOptionIds.CALL_SCENE) {
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                oDeviceObject = dss.staticDataModel.getDevice(dSUID);
                var iGroup = oDeviceObject.GroupID;
                if (iGroup == DSS.Groups.JOKER) {
                    iGroup = oDeviceObject.buttonGroupMembership;
                }
                this.parent.constructGridLvlDeviceScene(oDeviceObject.ZoneID, iGroup);
            } else if (this.parent.grids.lvl1.getSelected('_id') == actionSelectorLvl1OptionIds.SINGLE_DEVICE) {
                dSUID = this.parent.grids.lvlDevice.getSelected('dSUID');
                if (dSUID != null) {
                    switch (id) {
                        case actionSelectorDeviceActionOptionIds.OFF: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_SCENE,
                                params: {
                                    dsuid: dSUID,
                                    scene: DSS.Scenes.MIN,
                                    force: false
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.ON: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_SCENE,
                                params: {
                                    dsuid: dSUID,
                                    scene: DSS.Scenes.MAX,
                                    force: false
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LOCAL_ON: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_SCENE,
                                params: {
                                    dsuid: dSUID,
                                    scene: DSS.Scenes.LOCAL_ON,
                                    force: false
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.BLINK: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_BLINK,
                                params: {
                                    dsuid: dSUID
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_0_OF_3: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 0
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_1_OF_3: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 128
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_2_OF_3: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 255
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_0_OF_4: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 0
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_1_OF_4: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 96
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_2_OF_4: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 160
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.LEVEL_3_OF_4: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_VALUE,
                                params: {
                                    dsuid: dSUID,
                                    value: 255
                                }
                            };
                            break;
                        }
                        case actionSelectorDeviceActionOptionIds.IMPULSE: {
                            oObject = {
                                type: DSS.ActionTypes.DEVICE_SCENE,
                                params: {
                                    dsuid: dSUID,
                                    scene: DSS.Scenes.IMPULSE,
                                    force: false
                                }
                            };
                            break;
                        }
                    }
                    if (oObject) {
                        this.parent.setSelectedSubItem(oObject, true);
                    }
                }
            }
        });

        this.grids.lvlDeviceAction.on('itemdblclick', function () {     // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlDeviceAction);

    },
    selectColor: function (color) {
        this.selectedColor = color;
        document.getElementById('colorDisplay').style.backgroundColor = color;
        var rgb = hex2rgb(color);
        var hsv = rgb2hsv(rgb);
        var dSUID = this.grids.lvlDevice.getSelected('dSUID');
        var oObject = {
            type: DSS.ActionTypes.DEVICE_MOC_VALUE,
            params: {
                dsuid: dSUID,
                'channelid-0': dss.mocChannelIds.BRIGHTNESS,
                'channelvalue-0': Math.floor(hsv.bright * 255),
                'channelid-1': dss.mocChannelIds.HUE,
                'channelvalue-1': Math.floor(hsv.hue * 255),
                'channelid-2': dss.mocChannelIds.SATURATION,
                'channelvalue-2': Math.floor(hsv.sat * 255)
            }
        };
        this.setSelectedSubItem(oObject, true);
    },
    selectColorTemperatur: function (temperature, brightness) {
        this.selectedColTemperature = {
            temperature: temperature,
            bright: brightness
        };
        temperatureChannel = dss.miredToMOCValue(dss.kelvinToMired(this.selectedColTemperature.temperature));
        brightnessChannel = Math.round(this.selectedColTemperature.bright * 2.55);
        document.getElementById('colorTempDisplayTemperature').innerHTML = temperature + 'K';
        document.getElementById('colorTempDisplayBrightness').innerHTML = brightness + '%';
        var dSUID = this.grids.lvlDevice.getSelected('dSUID');
        var oObject = {
            type: DSS.ActionTypes.DEVICE_MOC_VALUE,
            params: {
                dsuid: dSUID,
                'channelid-0': dss.mocChannelIds.BRIGHTNESS,
                'channelvalue-0': brightnessChannel,
                'channelid-1': dss.mocChannelIds.COLOR_TEMPERATURE,
                'channelvalue-1': temperatureChannel
            }
        };
        this.setSelectedSubItem(oObject, true);
    },
    calculateWidthHelper: function (styleClass) {
        var el = document.getElementById('WidthHelper');
        if (!el) {
            el = document.createElement('DIV');
            el.id = 'WidthHelper';
            el.className = styleClass;
            el.style.position = 'fixed';
            el.style.top = '0px';
            el.style.left = '0px';
            el.style.marginLeft = '-99999px';
            document.body.appendChild(el);
        }
        return function (str) {
            el.innerHTML = str;
            return el.offsetWidth;
        };
    },
    constructGridLvlDevice: function (zoneID) {
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', { name: 'name', sortType: 'asUCText' }, 'icon', 'present' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDeviceStore = this.getDeviceStore(zoneID);
        oDeviceStore.each(function (record) {
            var oNewRecord = record.copy();
            oDeviceControlStore.add(oNewRecord);
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oNewRecord.get('name')));
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlDevice = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth + 21),    // + 21 for icon width
            hideHeaders: true,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDevice, this.actionSelectorName + 'GridLvlSingleDeviceZoneDevice');
        this.grids.lvlDevice.parent = this;
        this.grids.lvlDevice.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlDevice.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionSub);
            var dSUID = record.get('dSUID');
            this.parent.constructGridLvlDeviceActions(dSUID);
        });
        this.macPanel.add(this.grids.lvlDevice);
    },
    constructGridLvlGroupScene: function (zoneID, groupID, fAll, fForce) {
        var oSceneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'name', 'icon', 'reachable', 'checkable', 'checked', 'force', 'passive' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (fAll == null) {
            fAll = false;
        }
        if (fForce == null) {
            fForce = false;
        }
        var oSceneStore = this.getSceneStore(zoneID, groupID);
        if (groupID == DSS.Groups.TEMPERATURE && !this.fHeatingExtendedOperationMode) {

        } else {
            oSceneControlStore.add(
                new ActionGridShowAllCheckbox({ checked: fAll })
            );
            oSceneControlStore.add(
                new ActionGridSeparator({ reachable: true })
            );
        }
        fillAndAmendSceneStoreWithAllOff({
            sceneStore: oSceneStore,
            storeToFillAndAmend: oSceneControlStore,
            allScenesVisible: fAll,
            sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
            excludedGroups: [ DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],
            groupID: groupID
        });

        var localWidth = 324;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 3 * this.iDefaultWidth - 2;
        }

        this.grids.lvlScene = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            hideHeaders: true,
            viewConfig: {
                stripeRows: true
            },
            store: oSceneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlScene, this.actionSelectorName + 'GridLvlZoneGroupScene');
        this.grids.lvlScene.parent = this;
        this.grids.lvlScene.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlScene.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var groupID = this.parent.grids.lvlGroup.getSelected('group');
            var oObject;
            if (record.get('checkable') != null) {
                if (record.get('checkable')) {
                    var fChecked = !record.get('checked');
                    record.set('checked', fChecked);
                    grid.getSelectionModel().deselectAll();
                    if (record.get('id') == actionSelectorConsts.SHOW_ALL_GRID_ITEMS_ID) {
                        // alle Stimmungen
                        grid.store.removeAll();

                        grid.store.add(
                            new ActionGridShowAllCheckbox({ checked: record.get('checked') })
                        );
                        grid.store.add(
                            new ActionGridSeparator({ reachable: true })
                        );
                        fillAndAmendSceneStoreWithAllOff({
                            sceneStore: oSceneStore,
                            storeToFillAndAmend: grid.store,
                            allScenesVisible: fChecked,
                            sceneIdLimit: DSS.Scenes.ZONE_SCENES_END_AT,
                            excludedGroups: [ DSS.Groups.APARTMENT_VENTILATION, DSS.Groups.VENTILATION, DSS.Groups.RECIRCULATION ],  // TODO: ask if APARTMENT_VENTILATION should be excluded everywhere
                            groupID: groupID
                        });
                    }
                }
            }
            if (record.get('id') >= 0) {
                if (this.parent.grids.lvl1.getSelected('_id') == 0) {
                    var zoneID = this.parent.grids.lvlRoom.getSelected('ZoneID');
                    if (zoneID != null && groupID != null) {
                        if (groupID == DSS.Groups.TEMPERATURE) {
                            oObject = {
                                type: DSS.ActionTypes.HEATING_MODE,
                                params: {
                                    zone: zoneID,
                                    mode: record.get('id')
                                }
                            };
                            if (record.get('id') > actionSelectorConsts.RESET_MODE_SHIFT) {
                                oObject.params.mode = record.get('id') - actionSelectorConsts.RESET_MODE_SHIFT;
                                oObject.params.reset = true;
                            }
                        } else {
                            var fForce = record.get('force') || false;
                            oObject = {
                                type: DSS.ActionTypes.ZONE_SCENE,
                                params: {
                                    zone: zoneID,
                                    group: groupID,
                                    scene: record.get('id'),
                                    force: fForce
                                }
                            };
                        }
                        this.parent.setSelectedSubItem(oObject, true);
                    }
                }
            }
        });
        this.grids.lvlScene.on('itemdblclick', function () {    // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlScene);
    },
    constructGridLvlRoom: function () {
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', { name: 'name', sortType: 'asUCText' } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('actionConfigurator_zoneStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != DSS.Zones.ALL_DEVICES) {
                oZoneControlStore.add(oNewRecord);
            }
        });

        oZoneControlStore.add([ { name: _('All zones'), ZoneID: DSS.Zones.ALL_DEVICES } ]);
        oZoneControlStore.sort('name');
        this.grids.lvlRoom = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            hideHeaders: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record) {        // , colIdx, store, view
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr> ' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlRoom, this.actionSelectorName + 'GridLvlZone');

        this.grids.lvlRoom.parent = this;
        this.grids.lvlRoom.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlRoom.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionSub);
            var zoneID = record.get('ZoneID');
            this.parent.constructGridLvlGroup(zoneID);
        });
        this.macPanel.add(this.grids.lvlRoom);
    },
    constructGridLvlRoomDevice: function () {
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', { name: 'name', sortType: 'asUCText' } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('actionConfigurator_zoneDeviceStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != DSS.Zones.ALL_DEVICES) {
                oZoneControlStore.add(oNewRecord);
            }
        });


        oZoneControlStore.add([ { name: _('All zones'), ZoneID: DSS.Zones.ALL_DEVICES } ]);
        oZoneControlStore.sort('name');
        this.grids.lvlRoom = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            hideHeaders: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record) {    // , colIdx, store, view
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlRoom, this.actionSelectorName + 'GridLvlSingleDeviceZone');

        this.grids.lvlRoom.parent = this;
        this.grids.lvlRoom.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlRoom.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionSub);
            var zoneID = record.get('ZoneID');
            this.parent.constructGridLvlDevice(zoneID);
        });
        this.macPanel.add(this.grids.lvlRoom);
    },


    constructGridLvlHigh: function () {

        var localWidth = 624;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 1 * this.iDefaultWidth - 2;
        }

        var oStore = Ext.create('Ext.data.Store', {
            fields: [ { name: 'name', sortType: 'asUCText' }, 'id' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (dss.staticDataModel.HLECache) {
            for (var i = 0; i < dss.staticDataModel.HLECache.length; i++) {
                oStore.add({
                    name: dss.staticDataModel.HLECache[i].name,
                    id: dss.staticDataModel.HLECache[i].id
                });
            }
        }
        oStore.sort('name', 'ASC');
        this.grids.lvlHighLevelEvent = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            hideHeaders: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStore,
            columns: [ {
                sortable: true,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record) {        // , colIdx, store, view
                    const name = record.get('name');
                    return '<table data-id="' + record.get('id') + '" data-name="' + joinAndCapitalize(name) + '">' +
                        '<tr style="height:16px;"><td width="100%">' + name + '</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlHighLevelEvent, this.actionSelectorName + 'GridLvlUserDefinedAction');
        this.grids.lvlHighLevelEvent.parent = this;
        this.macPanel.add(this.grids.lvlHighLevelEvent);

        this.grids.lvlHighLevelEvent.on('itemdblclick', function () {   // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });
        this.grids.lvlHighLevelEvent.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlHighLevelEvent.on('itemclick', function (grid, record) {
            var oObject = {
                type: DSS.ActionTypes.CUSTOM_EVENT,
                params: {
                    event: record.get('id')
                }
            };
            this.parent.setSelectedSubItem(oObject, true);
        });

    },
    constructGridLvlURL: function () {

        var localWidth = 622;
        if (this.fUseAcutalWidth) {
            localWidth = this.getWidth() - 1 * this.iDefaultWidth - 2;
        }

        this.grids.lvlURLText = Ext.create('Ext.form.field.Text', {
            xtype: 'textfield',
            name: 'url',
            margin: 5,
            fieldLabel: 'URL',
            allowBlank: false,
            blankText: _('This field is required'),
            invalidText: _('The value in this field is invalid'),
            labelWidth: 30,
            width: localWidth - 20,
            enableKeyEvents: true,
            listeners: {
                keyup: function (edit) {
                    edit.parent.oActualValue = null;
                    if (edit.isValid()) {
                        var oObject = {
                            type: DSS.ActionTypes.URL,
                            params: {
                                url: edit.getValue()
                            }
                        };
                        edit.parent.setSelectedSubItem(oObject, true);
                        setTimeout((function () {
                            this.macPanel.el.dom.querySelector('[data-id="actionsGridLvlUrl"] input').focus();
                        }).bind(edit.parent), 0);
                    }
                }
            },
            valueToRaw: function (value) {
                if (value == null) {
                    return '';
                }
                value = value.replace(/&amp;/g, '&');
                return unescape(value);
            },
            rawToValue: function (value) {
                if (value == '') {
                    return null;
                }
                return escape(value);
            }
        });
        addDataIdToElement(this.grids.lvlURLText, 'Url', 'content');
        this.grids.lvlURL = Ext.create('Ext.panel.Panel', {
            height: this.iDefaultHeight,
            width: localWidth,
            autoScroll: true,
            items: [ this.grids.lvlURLText ]
        });
        addDataIdToElement(this.grids.lvlURL, this.actionSelectorName + 'GridLvlUrl');

        this.grids.lvlURL.parent = this;
        this.grids.lvlURLText.parent = this;
        this.macPanel.add(this.grids.lvlURL);
    },
    constructGridLvlGroup: function (zoneID) {
        var oGroupControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'id', 'group', 'name', 'icon', 'reachable', 'passive' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oGroupStore = this.getGroupStore(zoneID);
        oGroupStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable')) {
                var fOK = true;
                switch (oNewRecord.get('group')) {
                    case DSS.Groups.ALL:
                    case DSS.Groups.SECURITY:
                    case DSS.Groups.ACCESS:
                    case DSS.Groups.JOKER:
                    case 16:
                    case 17:
                    case 18:
                    case 19:
                    case 20:
                    case 21:
                    case 22:
                    case 23: {
                        fOK = false;
                        break;
                    }
                }
                if (fOK) {
                    oGroupControlStore.add(oNewRecord);
                }
                if (oNewRecord.get('group') == DSS.Groups.LIGHT) {
                    oGroupControlStore.add({
                        id: -1,         // todo: why -1?
                        group: null,
                        name: oNewRecord.get('name') + ' ' + _('blinking'),
                        icon: dss.staticDataModel.baseImagePath + 'sceneIcons/scene_blink.png',
                        reachable: oNewRecord.get('reachable')
                    });
                }
            }
        });
        oGroupControlStore.add({
            group: actionSelectorConsts.SEPARATOR_GRID_ITEM_ID,
            name: '-',
            icon: null,
            reachable: true,
            passive: true
        });

        var oSceneGroup0Store = null;
        // if (zoneID != DSS.Zones.ALL_DEVICES) {
        oSceneGroup0Store = this.getSceneStore(zoneID, 0);
        /* } else {
                var zoneArray = dss.staticDataModel.getZoneArray();
                if (zoneArray) {
                    for (var i=0;i<zoneArray.length;i++) {
                        var firstZone=dss.staticDataModel.getZoneArray()[i];
                        if (firstZone.ZoneID != DSS.Zones.ALL_DEVICES) {
                            oSceneGroup0Store = this.getSceneStore(firstZone.ZoneID, 0);
                            i=dss.staticDataModel.getZoneArray().length;
                        }
                    }
                }
            }*/
        if (oSceneGroup0Store) {
            oSceneGroup0Store.each(function (record) {
                var oNewRecord = record.copy();
                if (oNewRecord.get('reachable')) {
                    oGroupControlStore.add(oNewRecord);
                }
            });
        }

        this.grids.lvlGroup = Ext.create('DSS.grid.Panel', {
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            hideHeaders: true,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oGroupControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlGroup, this.actionSelectorName + 'GridLvlZoneGroup');
        this.grids.lvlGroup.on('itemdblclick', function () {    // grid, record
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });
        this.grids.lvlGroup.on('beforeitemclick', function (grid, record) {
            return !record.get('passive');
        });
        this.grids.lvlGroup.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            var oObject;
            if (this.parent.grids.lvl1.getSelected('_id') == 0) {
                var zoneID = this.parent.grids.lvlRoom.getSelected('ZoneID');
                var groupID = record.get('group');
                if (zoneID != null && groupID != null) {
                    this.parent.constructGridLvlGroupScene(zoneID, groupID);
                } else {
                    var id = record.get('id');
                    if (id != null) {
                        if (id < 0) {
                            oObject = {
                                type: DSS.ActionTypes.ZONE_BLINK,
                                params: {
                                    zone: zoneID,
                                    group: Math.abs(id)
                                }
                            };
                            this.parent.setSelectedSubItem(oObject, true);
                        } else {
                            oObject = {
                                type: DSS.ActionTypes.ZONE_SCENE,
                                params: {
                                    zone: zoneID,
                                    group: DSS.Groups.ALL,
                                    scene: id,
                                    force: false
                                }
                            };
                            this.parent.setSelectedSubItem(oObject, true);
                        }
                    }
                }
            }
        });


        this.grids.lvlGroup.parent = this;
        this.macPanel.add(this.grids.lvlGroup);
    },
    constructMacGrids: function () {
        var lvl1Store = Ext.data.StoreManager.lookup('actionConfigurator_l1Store');

        if (this.grids.lvl1 == null) {
            var lvl1TempStore = Ext.create('Ext.data.Store', {
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            lvl1Store.each(function (record) {
                var oNewRecord = record.copy();
                lvl1TempStore.add(oNewRecord);
            });
            this.grids.lvl1 = Ext.create('DSS.grid.Panel', {
                height: this.iDefaultHeight,
                width: this.iDefaultWidth,
                hideHeaders: true,
                autoScroll: true,
                viewConfig: {
                    stripeRows: true
                },
                store: lvl1TempStore,
                columns: [ {
                    sortable: false,
                    hideable: false,
                    dataIndex: 'name',
                    flex: 1,
                    renderer: this.defMacGridRenderer
                } ],

                findStoreItem: function (fieldName, value, startIndex, anyMatch, caseSensitive, exactMatch) {
                    return this.store.find(fieldName, value, startIndex, anyMatch, caseSensitive, exactMatch);
                }
            });
            addDataIdToElement(this.grids.lvl1, this.actionSelectorName + 'GridLvl1');
            this.grids.lvl1.parent = this;
            this.grids.lvl1.on('itemdblclick', function () {    // grid, record
                if (this.parent.oActualValue != null) {
                    this.parent.macPanel.collapse();
                }
            });

            this.grids.lvl1.on('beforeitemclick', function (grid, record) {
                return !record.get('passive');
            });
            this.grids.lvl1.on('itemclick', function (grid, record) {
                for (var sKey in this.parent.grids) {
                    if (sKey !== 'lvl1') {
                        this.parent.grids[sKey] && this.parent.macPanel.remove(this.parent.grids[sKey]);
                    }
                }
                this.parent.oActualValue = null;
                var fIsPassive = false;
                if (record.get('passive') == true) {
                    this.getSelectionModel().deselect(record);
                    fIsPassive = true;
                }

                if (!fIsPassive) {
                    // build second column, depending on which option is selected in the first:
                    const id = record.get('_id');
                    if (id === actionSelectorLvl1OptionIds.ACTIVITY_IN_ROOM) {
                        this.parent.constructGridLvlRoom();
                    } else if (id === actionSelectorLvl1OptionIds.SINGLE_DEVICE) {
                        this.parent.constructGridLvlRoomDevice();
                    } else if (id === actionSelectorLvl1OptionIds.USER_DEFINED_ACTIONS) {
                        this.parent.constructGridLvlHigh();
                    } else if (id === actionSelectorLvl1OptionIds.CALL_URL) {
                        this.parent.constructGridLvlURL();
                    } else if (id === actionSelectorLvl1OptionIds.ACCESS) {
                        this.parent.constructGridLvlAccess();
                    } else if (id === actionSelectorLvl1OptionIds.WEATHER) {
                        this.parent.constructGridLvlWeather();
                    } else if (id === actionSelectorLvl1OptionIds.SECURITY) {
                        this.parent.constructGridLvlPanic();
                    } else if (id === actionSelectorLvl1OptionIds.USER_GROUP) {
                        this.parent.constructGridLvlUserGroup();
                    } else if (id === actionSelectorLvl1OptionIds.USER_STATES) {
                        this.parent.constructGridLvlUserState();
                    } else if (id === actionSelectorLvl1OptionIds.HEATING) {
                        this.parent.constructGridLvlHeating();
                    } else if (id === actionSelectorLvl1OptionIds.AUDIO) {
                        this.parent.constructGridLvlAudio();
                    }
                }
                this.parent.macPanel.doLayout();
            });
        } else {
            this.grids.lvl1.getStore().removeAll();
            var mThis = this;
            lvl1Store.each(function (record) {
                var oNewRecord = record.copy();
                mThis.grids.lvl1.getStore().add(oNewRecord);
            });
        }
    },
    setSelectedSubItem: function (oObject, fNoMacgridRebuild) {
        this.oActualValue = oObject;
        const mainRecord = this.getSelectedMainRecord();
        if (mainRecord == null) {
            var oNewRecord = this.gridStore.add({
                action: oObject,
                delay: 0,
                delayHr: 0,
                delayMin: 0,
                delaySec: 0
            });
            var i = this.gridStore.indexOf(oNewRecord[0]);
            this.setSelectedMainRecord(i, fNoMacgridRebuild);
        } else if (oObject) {
            mainRecord.set('action', oObject);
            mainRecord.commit();
        }

        this.gridStore.sort('delay', 'ASC');
        this.removeCls('myInvalid');
        this.mainGrid.doComponentLayout();
        if (!this.macPanel.collapsed) {
            this.updatePanelDescription();
        }
    },
    setSelectedMainRecord: function (index, fNoMacgridRebuild) {
        if (this.fDelayEnabled) {
            this.mainGrid.getSelectionModel().setCurrentPosition({ row: index, column: 0 });
        } else {
            this.mainGrid.getSelectionModel().select(index);
        }
        if (fNoMacgridRebuild != null) {
            if (fNoMacgridRebuild) {
                return;
            }
        }
        this.setMacGrid(this.getSelectedMainRecord().get('action'));
    },
    getSelectedMainRecord: function () {
        if (this.fDelayEnabled) {
            if (this.mainGrid.getSelectionModel().getCurrentPosition() == null) {
                return null;
            }
            return this.mainGrid.getStore().getAt(this.mainGrid.getSelectionModel().getCurrentPosition().row);
        } else {
            if (this.mainGrid.getSelected() == null) {
                return null;
            }
            return this.mainGrid.getSelected();
        }
    },
    deselectMainGrid: function () {
        var oldSelect = this.getSelectedMainRecord();
        if (oldSelect != null) {
            if (this.fDelayEnabled) {
                this.mainGrid.getSelectionModel().setCurrentPosition(null);
            } else {
                this.mainGrid.getSelectionModel().deselect(oldSelect);
            }
        }
    },
    setMacGrid: function (oObject) {
        this.selectedColor = null;
        this.selectedColTemperature = null;
        var sKey;
        for (sKey in this.grids) {
            if (this.grids.hasOwnProperty(sKey) && sKey !== 'lvl1') {
                this.grids[sKey] && this.macPanel.remove(this.grids[sKey]);
            }
        }
        if (oObject) {
            this.oActualValue = oObject;
        }
        if (oObject == null) {
            var oTempGrid = this.grids.lvl1;
            oTempGrid.store.removeAll();
            var lvl1Store = Ext.data.StoreManager.lookup('actionConfigurator_l1Store');
            lvl1Store.each(function (record) {
                var oNewRecord = record.copy();
                oTempGrid.store.add(oNewRecord);
            });
            this.grids.lvl1.getSelectionModel().deselectAll();
            // this.grids.lvl1.getSelectionModel().select(0);
            // this.constructGridLvlRoom();
        } else {
            var oDevice;
            var zoneID;
            var groupID;
            var oRecordScene;
            var fn;
            var me = this;
            switch (oObject.type) {
                case DSS.ActionTypes.CHANGE_ADDON_STATE: {
                    if (oObject.params.addonId === 'system-addon-user-defined-states') {
                        this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.USER_STATES);
                        this.constructGridLvlUserState();
                        if (this.grids.lvlUserState.addStoreItemToSelection('_id', oObject.params.statename)) {
                            this.constructGridLvlUserStateValue(oObject.params.statename);
                            this.grids.lvlUserStateValue.addStoreItemToSelection('value', oObject.params.state);
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.CHANGE_STATE: {
                    // oAction.params.state
                    // if (oObject.params.statename=='heating_water_system') {
                    //     if (this.fHighLevel)
                    //         this.grids.lvl1.getSelectionModel().select(9);
                    //     else
                    //         this.grids.lvl1.getSelectionModel().select(8);
                    //     this.constructGridLvlHeating();
                    //     var sID="";
                    //     if (oObject.params.state=="cold water")
                    //         sID='heating_mode.cooling';
                    //     if (oObject.params.state=="hot water")
                    //         sID='heating_mode.heating';
                    //     if (oObject.params.state=="off")
                    //         sID='heating_mode.off';
                    //     oRecordScene = this.grids.lvlHeating.store.find('id', sID,0,false,false,true);
                    //     if (oRecordScene != -1)
                    //         this.grids.lvlHeating.getSelectionModel().select(oRecordScene,true);
                    // } else
                    if (oObject.params.statename === 'fire') {
                        this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SECURITY);
                        this.constructGridLvlPanic();
                        this.grids.lvlPanic.addStoreItemToSelection('id', oObject.params.statename + '_' + oObject.params.state);
                    } else {
                        this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.WEATHER);
                        this.constructGridLvlWeather();
                        oRecordScene = this.grids.lvlWeather.addStoreItemToSelection('id', oObject.params.statename + '_' + oObject.params.state);
                    }
                    break;
                }

                case DSS.ActionTypes.UNDO_ZONE_SCENE: {
                    if (oObject.params.zone == DSS.Zones.ALL_DEVICES) {
                        switch (oObject.params.scene) {
                            case DSS.Scenes.PANIC:
                            case DSS.Scenes.FIRE:
                            case DSS.Scenes.ALARM_1:
                            case DSS.Scenes.ALARM_2:
                            case DSS.Scenes.ALARM_3:
                            case DSS.Scenes.ALARM_4: {
                                this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SECURITY);
                                this.constructGridLvlPanic();
                                this.grids.lvlPanic.addStoreItemToSelection('id', actionSelectorConsts.UNDO_SCENE_SHIFT + oObject.params.scene);
                                return;
                            }
                        }
                    }
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.ACTIVITY_IN_ROOM);
                    this.constructGridLvlRoom();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', oObject.params.zone)) {
                        break;
                    }
                    this.constructGridLvlGroup(oObject.params.zone);
                    if (oObject.params.group == DSS.Groups.ALL) {
                        if (!this.grids.lvlGroup.selectStoreItem('id', oObject.params.scene)) {
                            break;
                        }
                    } else {
                        if (!this.grids.lvlGroup.selectStoreItem('group', oObject.params.group)) {
                            break;
                        }
                        this.constructGridLvlGroupScene(oObject.params.zone, oObject.params.group, false, oObject.params.force);
                        oRecordScene = this.grids.lvlScene[
                            oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                        ]('id', oObject.params.scene);
                        if (oRecordScene === -1) {
                            this.grids.lvlScene && this.macPanel.remove(this.grids.lvlScene);
                            this.constructGridLvlGroupScene(oObject.params.zone, oObject.params.group, true, oObject.params.force);
                            oRecordScene = this.grids.lvlScene[
                                oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                            ]('id', -1 * oObject.params.scene);
                            if (oRecordScene === -1) {
                                break;
                            }
                        }
                        if (oRecordScene > DSS.Zones.ZONE_SCENES_START_AT) {
                            fn = function () {
                                me.grids.lvlScene.setScrollTop(oRecordScene * 21);
                            };
                            window.setTimeout(fn, 10);
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.HEATING_MODE: {
                    oObject.params.group = DSS.Groups.TEMPERATURE;
                    oObject.params.scene = oObject.params.mode;
                    if (oObject.params.reset) {
                        oObject.params.scene += actionSelectorConsts.RESET_MODE_SHIFT;
                    }
                    // no 'break' here on purpose! grid build-up is the same as for the DSS.ActionTypes.ZONE_SCENE
                }

                case DSS.ActionTypes.ZONE_SCENE: {
                    if (oObject.params.zone == DSS.Zones.ALL_DEVICES) {
                        if (dss.staticDataModel.isClusterGroup(oObject.params.group)) {
                            this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.USER_GROUP);
                            this.constructGridLvlUserGroup();
                            const userGroupFound = this.grids.lvlUserGroup.selectStoreItem('id', oObject.params.group);
                            if (userGroupFound) {
                                this.constructGridLvlUserGroupScene(oObject.params.group);
                                oRecordScene = this.grids.lvlUsergroupScene[
                                    oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                                ]('id', oObject.params.scene);
                                if (oRecordScene === -1) {
                                    this.grids.lvlUsergroupScene && this.macPanel.remove(this.grids.lvlUsergroupScene);
                                    this.constructGridLvlUserGroupScene(oObject.params.group, true, oObject.params.force);
                                    oRecordScene = this.grids.lvlUsergroupScene[
                                        oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                                    ]('id', oObject.params.scene);
                                    if (oRecordScene === -1) {
                                        break;
                                    }
                                }
                                if (oRecordScene > DSS.Zones.ZONE_SCENES_START_AT) {
                                    fn = function () {
                                        me.grids.lvlUsergroupScene.setScrollTop(oRecordScene * 21);
                                    };
                                    window.setTimeout(fn, 10);
                                }
                            }
                            break;
                        }

                        switch (oObject.params.scene) {
                            case DSS.Scenes.PANIC:
                            case DSS.Scenes.FIRE:
                            case DSS.Scenes.ALARM_1:
                            case DSS.Scenes.ALARM_2:
                            case DSS.Scenes.ALARM_3:
                            case DSS.Scenes.ALARM_4: {
                                this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SECURITY);
                                this.constructGridLvlPanic();
                                this.grids.lvlPanic.addStoreItemToSelection('id', oObject.params.scene);
                                this.updatePanelDescription();
                                return;
                            }
                            case DSS.Scenes.RAIN:
                            case DSS.Scenes.NO_RAIN: {
                                this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.WEATHER);
                                this.constructGridLvlWeather();
                                this.grids.lvlWeather.addStoreItemToSelection('id', oObject.params.scene);
                                this.updatePanelDescription();
                                return;
                            }
                            case DSS.Scenes.DOORBELL:
                            case DSS.Scenes.ABSENT:
                            case DSS.Scenes.PRESENT: {
                                this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.ACCESS);
                                this.constructGridLvlAccess();
                                this.grids.lvlAccess.addStoreItemToSelection('id', oObject.params.scene);
                                this.updatePanelDescription();
                                return;
                            }
                        }
                    }
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.ACTIVITY_IN_ROOM);
                    this.constructGridLvlRoom();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', oObject.params.zone)) {
                        break;
                    }
                    this.constructGridLvlGroup(oObject.params.zone);
                    if (oObject.params.group == DSS.Groups.ALL) {
                        if (!this.grids.lvlGroup.selectStoreItem('id', oObject.params.scene)) {
                            break;
                        }
                    } else {
                        if (!this.grids.lvlGroup.selectStoreItem('group', oObject.params.group)) {
                            break;
                        }
                        this.constructGridLvlGroupScene(oObject.params.zone, oObject.params.group, false, oObject.params.force);
                        oRecordScene = this.grids.lvlScene[
                            oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                        ]('id', oObject.params.scene);
                        if (oRecordScene === -1) {
                            this.grids.lvlScene && this.macPanel.remove(this.grids.lvlScene);
                            this.constructGridLvlGroupScene(oObject.params.zone, oObject.params.group, true, oObject.params.force);
                            oRecordScene = this.grids.lvlScene[
                                oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                            ]('id', oObject.params.scene);
                            if (oRecordScene === -1) {
                                break;
                            }
                        }
                        if (oRecordScene > DSS.Zones.ZONE_SCENES_START_AT) {
                            fn = function () {
                                if (typeof me.grids.lvlScene.setScrollTop === 'function') {
                                    me.grids.lvlScene.setScrollTop(oRecordScene * 21);
                                }
                            };
                            window.setTimeout(fn, 10);
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.DEVICE_MOC_VALUE: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SINGLE_DEVICE);
                    oDevice = dss.staticDataModel.getDevice(oObject.params.dsuid);
                    zoneID = oDevice.ZoneID;
                    if (typeof zoneID === 'undefined') {
                        break;
                    }
                    this.constructGridLvlRoomDevice();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', zoneID)) {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    if (!this.grids.lvlDevice.selectStoreItem('dSUID', oObject.params.dsuid)) {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oObject.params.dsuid);

                    const hexColor = dss.mocToHexColor(oObject.params);
                    if (hexColor) {
                        this.selectedColor = hexColor;
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.COLOR);
                    }
                    const color = dss.mocToBrightnessTemperatureColor(oObject.params);
                    if (color) {
                        this.selectedColTemperature = color;
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.COLOR_TEMPERATURE);
                    }
                    break;
                }

                case DSS.ActionTypes.DEVICE_SCENE: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SINGLE_DEVICE);
                    oDevice = dss.staticDataModel.getDevice(oObject.params.dsuid);
                    zoneID = oDevice.ZoneID;
                    if (typeof zoneID === 'undefined') {
                        break;
                    }
                    groupID = oDevice.GroupID;
                    this.constructGridLvlRoomDevice();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', zoneID)) {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    if (!this.grids.lvlDevice.selectStoreItem('dSUID', oObject.params.dsuid)) {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oObject.params.dsuid);
                    const specialActions = oDevice.specialActions.actions;
                    if (specialActions && specialActions.deviceActions) {
                        for (sKey in specialActions.deviceActions) {
                            const action = specialActions.deviceActions[sKey];
                            if (action.scene != null) {
                                if (action.scene == oObject.params.scene) {
                                    this.grids.lvlDeviceAction.selectStoreItem('id', Number(sKey) + actionSelectorConsts.SPECIAL_ACTION_SHIFT);
                                }
                            }
                        }
                        break;
                    }
                    if (oObject.params.scene == DSS.Scenes.MIN) {
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.OFF);
                    } else if (oObject.params.scene == DSS.Scenes.MAX) {
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.ON);
                    } else if (oObject.params.scene == DSS.Scenes.LOCAL_ON) {
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.LOCAL_ON);
                    } else if (oObject.params.scene == DSS.Scenes.IMPULSE) {     // && isImpulseCapableJoker(oDevice)
                        this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.IMPULSE);
                    } else {
                        if (!this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.CALL_SCENE)) {
                            break;
                        }
                        this.constructGridLvlDeviceScene(zoneID, groupID);
                        oRecordScene = this.grids.lvlDeviceScene[
                            oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                        ]('id', oObject.params.scene);
                        if (oRecordScene === -1) {
                            this.grids.lvlDeviceScene && this.macPanel.remove(this.grids.lvlDeviceScene);
                            this.constructGridLvlDeviceScene(zoneID, groupID, true, oObject.params.force);
                            oRecordScene = this.grids.lvlDeviceScene[
                                oObject.params.force ? 'selectNextStoreItemAndReturnIndex' : 'selectStoreItemAndReturnIndex'
                            ]('id', oObject.params.scene);
                        }
                        if (oRecordScene > DSS.Zones.ZONE_SCENES_START_AT) {
                            fn = function () {
                                me.grids.lvlDeviceScene.setScrollTop(oRecordScene * 21);
                            };
                            window.setTimeout(fn, 10);
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.DEVICE_ACTION: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SINGLE_DEVICE);
                    oDevice = dss.staticDataModel.getDevice(oObject.params.dsuid);
                    zoneID = oDevice.ZoneID;
                    if (typeof zoneID === 'undefined') {
                        break;
                    }
                    this.constructGridLvlRoomDevice();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', zoneID)) {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    if (!this.grids.lvlDevice.selectStoreItem('dSUID', oObject.params.dsuid)) {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oObject.params.dsuid);
                    if (!this.grids.lvlDeviceAction.selectStoreItem('id', 'actions_' + oObject.params.id)) {
                        var actionStandard = dss.staticDataModel.getStandardActions(oObject.params.dsuid);
                        var actionCustom = dss.staticDataModel.getCustomActions(oObject.params.dsuid);
                        var oDeviceActions = dss.staticDataModel.getDevice(oObject.params.dsuid).deviceActions;
                        var category = null;

                        for (var sKey in actionStandard) {
                            if (sKey == oObject.params.id) {
                                category = actionStandard[sKey].description.category;
                            }
                        }
                        for (var sKey in actionCustom) {
                            if (sKey == oObject.params.id) {
                                category = actionCustom[sKey].description.category;
                            }
                        }
                        for (var sKey in oDeviceActions) {
                            if (oDeviceActions.hasOwnProperty(sKey) && sKey === oObject.params.id) {
                                category = oDeviceActions[sKey].category;
                            }
                        }

                        if (category != null) {
                            if (this.grids.lvlDeviceAction.selectStoreItem('id', 'category_' + category)) {
                                this.constructGridLvlDeviceCategoryAction(oObject.params.dsuid, category);
                                this.grids.lvlDeviceActionSub.selectStoreItem('id', 'actions_' + oObject.params.id);
                            }
                        }
                    }
                    break;
                }

                case DSS.ActionTypes.DEVICE_VALUE: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SINGLE_DEVICE);
                    oDevice = dss.staticDataModel.getDevice(oObject.params.dsuid);
                    zoneID = oDevice.ZoneID;
                    if (typeof zoneID === 'undefined') {
                        break;
                    }
                    this.constructGridLvlRoomDevice();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', zoneID)) {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    if (!this.grids.lvlDevice.selectStoreItem('dSUID', oObject.params.dsuid)) {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oObject.params.dsuid);
                    const specialActions = oDevice.specialActions.actions;
                    if (specialActions && specialActions.deviceActions) {
                        for (sKey in specialActions.deviceActions) {
                            const action = specialActions.deviceActions[sKey];
                            if (action.outputValue != null) {
                                if (action.outputValue == oObject.params.value) {
                                    this.grids.lvlDeviceAction.selectStoreItem('id', Number(sKey) + actionSelectorConsts.SPECIAL_ACTION_SHIFT);
                                }
                            }
                        }
                        break;
                    }
                    let deviceAction;
                    if (oDevice.outputMode == 34) {
                        switch (oObject.params.value) {
                            case 0:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_0_OF_3;
                                break;
                            case 128:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_1_OF_3;
                                break;
                            case 255:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_2_OF_3;
                                break;
                        }
                    } else if (oDevice.outputMode == 38) {
                        switch (oObject.params.value) {
                            case 0:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_0_OF_4;
                                break;
                            case 96:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_1_OF_4;
                                break;
                            case 160:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_2_OF_4;
                                break;
                            case 255:
                                deviceAction = actionSelectorDeviceActionOptionIds.LEVEL_3_OF_4;
                                break;
                        }
                    }
                    if (deviceAction) {
                        this.grids.lvlDeviceAction.selectStoreItem('id', deviceAction);
                    }
                    break;
                }

                case DSS.ActionTypes.DEVICE_BLINK: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.SINGLE_DEVICE);
                    oDevice = dss.staticDataModel.getDevice(oObject.params.dsuid);
                    zoneID = oDevice.ZoneID;
                    if (typeof zoneID === 'undefined') {
                        break;
                    }
                    this.constructGridLvlRoomDevice();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', zoneID)) {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    if (!this.grids.lvlDevice.selectStoreItem('dSUID', oObject.params.dsuid)) {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oObject.params.dsuid);
                    this.grids.lvlDeviceAction.selectStoreItem('id', actionSelectorDeviceActionOptionIds.BLINK);
                    break;
                }

                case DSS.ActionTypes.ZONE_BLINK: {
                    if (oObject.params.zone == DSS.Zones.ALL_DEVICES) {
                        if (dss.staticDataModel.isClusterGroup(oObject.params.group)) {
                            this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.USER_GROUP);
                            this.constructGridLvlUserGroup();
                            this.grids.lvlUserGroup.selectStoreItem('id', -1 * oObject.params.group);
                            break;
                        }
                    }
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.ACTIVITY_IN_ROOM);
                    this.constructGridLvlRoom();
                    if (!this.grids.lvlRoom.selectStoreItem('ZoneID', oObject.params.zone)) {
                        break;
                    }
                    this.constructGridLvlGroup(oObject.params.zone);
                    this.grids.lvlGroup.selectStoreItem('id', -1 * oObject.params.group);
                    break;
                }

                case DSS.ActionTypes.CUSTOM_EVENT: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.USER_DEFINED_ACTIONS);
                    this.constructGridLvlHigh();
                    this.grids.lvlHighLevelEvent.selectStoreItem('id', oObject.params.event);
                    break;
                }

                case DSS.ActionTypes.URL: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.CALL_URL);
                    this.constructGridLvlURL();
                    this.grids.lvlURLText.setValue(oObject.params.url);
                    break;
                }
                case DSS.ActionTypes.AUDIO_ACTIVITY: {
                    this.grids.lvl1.selectStoreItem('_id', actionSelectorLvl1OptionIds.AUDIO);
                    this.constructGridLvlAudio();
                    this.grids.lvlAudio.selectStoreItem('id', oObject.params.contentSource);
                    break;
                }
            }
        }
        this.updatePanelDescription();
    },
    eventHandlerSelectionChange: function (view, selections) {  // , options
        var mThis = this;
        if (this.parent != null) {
            mThis = this.parent;
        }
        if (selections.length > 0) {
            mThis.oButtonEdit.setDisabled(false);
            mThis.oButtonDelete.setDisabled(false);
            var selIndex = -1;
            for (var i = 0; i < view.store.getCount(); i++) {
                if (view.store.getAt(i) == selections[0]) {
                    selIndex = i;
                }
            }
            mThis.oButtonUp.setDisabled(selIndex == 0);
            mThis.oButtonDown.setDisabled(selIndex == view.store.getCount() - 1);
            i = mThis.mainGrid.store.indexOf(selections[0]);
            mThis.lblSelectorAnzeige.setText(Ext.String.format2(_('activity {index}/{count}'), {
                index: (i + 1).toString(),
                count: view.store.getCount().toString()
            }), false);

            mThis.setSelectedMainRecord(i);
            mThis.oActualValue = mThis.getSelectedMainRecord().get('action');
        } else {
            mThis.oButtonEdit.setDisabled(true);
            mThis.oButtonDelete.setDisabled(true);
            mThis.oButtonUp.setDisabled(true);
            mThis.oButtonDown.setDisabled(true);
            mThis.lblSelectorAnzeige.setText(Ext.String.format2(_('{count} activity', '{count} activites', view.store.getCount()), {
                count: view.store.getCount().toString()
            }), false);

            mThis.oActualValue = null;
        }
    },
    constructor: function (config) {
        Ext.tip.QuickTipManager.init();
        var mThis = this;
        this.fHighLevel = true;
        this.fEnableUsergroup = false;
        this.fIgnorePresence = false;
        if (config.ignorePresence != null) {
            this.fIgnorePresence = config.ignorePresence;
        }
        this.fHeatingOperationMode = false;
        if (config.heatingOperationMode != null) {
            this.fHeatingOperationMode = config.heatingOperationMode;
        }
        this.fHeatingExtendedOperationMode = false;
        if (config.heatingExtendedOperationMode != null) {
            this.fHeatingExtendedOperationMode = config.heatingExtendedOperationMode;
        }
        if (config.noHighlevel != null) {
            if (config.noHighlevel) {
                this.fHighLevel = false;
            }
        }
        if (config.UMRExtendetFunctions != null) {
            this.fUMRExtendetFunctions = config.UMRExtendetFunctions;
        }
        if (config.userStates != null) {
            this.fUserstates = config.userStates;
        }
        this.fUserstates = true;

        this.fDelayEnabled = false;
        if (config.withDelay != null) {
            this.fDelayEnabled = config.withDelay;
        }
        if (config.allowNoAction != null) {
            this.fAllowNoAction = config.allowNoAction;
        }
        if (config.enableUsergroup != null) {
            this.fEnableUsergroup = config.enableUsergroup;
        }

        this.actionSelectorName = config.myID
            ? config.myID
            : config.autoEl && config.autoEl['data-id']
                ? config.autoEl['data-id']
                : '';

        this.oColorEditor = Ext.create(
            'DSS.component.ColorPicker', {
                clickable: false,
                listeners: {
                    setValue: function (newValue) {
                        mThis.selectColor(newValue);
                    }
                }
            }
        );

        this.oColTempEditor = Ext.create(
            'DSS.component.ColorTemperatureDialog', {
                clickable: false,
                listeners: {
                    setValue: function (temperature, brightness) {
                        mThis.selectColorTemperatur(temperature, brightness);
                    }
                }
            }
        );

        this.initStores(this.fHighLevel, this.fUserstates);
        this.initGridStore();
        Ext.data.StoreManager.on('clear', function () {
            mThis.initStores(mThis.fHighLevel, mThis.fUserstates);
            mThis.constructMacGrids();
        });

        this.constructMacGrids();
        this.lblSelectorAnzeige = Ext.create('Ext.form.Label', {
            width: 200,
            height: 22,
            padding: 4,
            style: {
                textAlign: 'right'
            }
        });

        this.oButtonNew = Ext.create('Ext.button.Button', {
            iconCls: 'icon-add-action',
            text: _('new activity'),
            handler: function () {
                mThis.oActualValue = null;
                if (mThis.macPanel.collapsed) {
                    mThis.macPanel.expand(true);
                } else {
                    mThis.setMacGrid();
                }
                mThis.deselectMainGrid();
            }
        });
        addDataIdToElement(this.oButtonNew, this.actionSelectorName + 'NewActionButton');
        this.oButtonNew.parent = this;

        this.oButtonEdit = Ext.create('Ext.button.Button', {
            iconCls: 'icon-edit-action',
            text: _('edit activity'),
            disabled: true,
            handler: function () {
                if (mThis.macPanel.collapsed) {
                    mThis.oActualValue = mThis.getSelectedMainRecord().get('action');
                    mThis.macPanel.expand(true);
                    // mThis.setMacGrid(mThis.oActualValue);
                } else {
                    mThis.macPanel.collapse();
                }
            }
        });
        addDataIdToElement(this.oButtonEdit, this.actionSelectorName + 'EditOrShowAllActionsButton');
        this.oButtonEdit.parent = this;

        this.oButtonUp = Ext.create('Ext.button.Button', {
            iconCls: 'icon-left-action',
            disabled: true,
            handler: function () {
                var i = mThis.mainGrid.store.indexOf(mThis.getSelectedMainRecord());
                mThis.setSelectedMainRecord(i - 1);
            }
        });
        addDataIdToElement(this.oButtonUp, this.actionSelectorName + 'ActionUpButton');
        this.oButtonUp.parent = this;

        this.oButtonDown = Ext.create('Ext.button.Button', {
            iconCls: 'icon-right-action',
            disabled: true,
            handler: function () {
                var i = mThis.mainGrid.store.indexOf(mThis.getSelectedMainRecord());
                mThis.setSelectedMainRecord(i + 1);
            }
        });
        addDataIdToElement(this.oButtonDown, this.actionSelectorName + 'ActionDownButton');
        this.oButtonDown.parent = this;

        this.oButtonDelete = Ext.create('Ext.button.Button', {
            iconCls: 'icon-delete-action',
            tooltip: _('delete activity'),
            disabled: true,
            handler: function () {
                mThis.macPanel.collapse();
                mThis.gridStore.remove(mThis.getSelectedMainRecord());
                mThis.macPanel.setTitle('');
            }
        });
        addDataIdToElement(this.oButtonDelete, this.actionSelectorName + 'DeleteActionButton');
        this.oButtonDelete.parent = this;

        if (this.fDelayEnabled) {
            // var selModel = Ext.create('Ext.selection.CellModel', {
            //     allowDeselect: true,
            //     mode: 'SINGLE'
            // });

            var cellEditing = Ext.create('Ext.grid.plugin.CellEditing', { clicksToEdit: 1 });
            const secondsMinutesEditor = {
                xtype: 'numberfield',
                allowBlank: false,
                blankText: _('This field is required'),
                invalidText: _('The value in this field is invalid'),
                minValue: 0,
                maxValue: 59,
                hideTrigger: true,
                keyNavEnabled: false,
                mouseWheelEnabled: false
            };

            this.mainGrid = Ext.create('DSS.grid.Panel', {
                cls: 'Y-action-cell',
                enableColumnHide: false,
                enableColumnMove: false,
                selType: 'cellmodel',
                border: '0',
                region: 'center',
                height: '100%',
                store: this.gridStore,
                hideHeaders: true,
                overflowX: 'hidden',
                plugins: [ cellEditing ],
                width: 772,
                viewConfig: {
                    stripeRows: true
                },
                // selModel: selModel,
                columns: [ {
                    header: _('action'),
                    dataIndex: 'action',
                    sortable: false,
                    hideable: false,
                    flex: 1,
                    renderer: this.defMainGridRenderer
                }, {
                    header: _('delay'),
                    dataIndex: 'delayHr',
                    sortable: false,
                    hideable: false,
                    width: 28,
                    align: 'right',
                    editor: {
                        xtype: 'numberfield',
                        allowBlank: false,
                        blankText: _('This field is required'),
                        invalidText: _('The value in this field is invalid'),
                        minValue: 0,
                        hideTrigger: true,
                        keyNavEnabled: false,
                        mouseWheelEnabled: false
                    }
                }, {
                    dataIndex: 'delayMin',
                    sortable: false,
                    hideable: false,
                    width: 2,
                    padding: '0 0 0 0',
                    renderer: function () {     // val, metadata, record, colIdx, store, view
                        return ':';
                    }
                }, {
                    dataIndex: 'delayMin',
                    sortable: false,
                    hideable: false,
                    width: 28,
                    renderer: function (val) {      // , metadata, record, colIdx, store, view
                        return val.toString().padStart(2, 0);
                    },
                    editor: secondsMinutesEditor
                }, {
                    dataIndex: 'delayMin',
                    sortable: false,
                    hideable: false,
                    width: 2,
                    renderer: function () {     // val, metadata, record, colIdx, store, view
                        return ':';
                    }
                }, {
                    dataIndex: 'delaySec',
                    sortable: false,
                    hideable: false,
                    width: 28,
                    renderer: function (val) {      // , metadata, record, colIdx, store, view
                        return val.toString().padStart(2, 0);
                    },
                    editor: secondsMinutesEditor
                } ],
                tbar: [ this.oButtonNew, this.oButtonEdit,
                // { xtype: 'button', text: 'Erweiterte Ansicht',
                // id: 'action-selector-btn-extended' ,
                // disabled:true},
                    '->', this.lblSelectorAnzeige, this.oButtonUp, this.oButtonDown, this.oButtonDelete ]
            });
        } else {
            this.mainGrid = Ext.create('DSS.grid.Panel', {
                enableColumnHide: false,
                enableColumnMove: false,
                selType: 'rowmodel',
                border: '0',
                region: 'center',
                // height : "100%",
                store: this.gridStore,
                hideHeaders: true,
                overflowX: 'hidden',
                columns: [ {
                    header: _('action'),
                    dataIndex: 'action',
                    sortable: false,
                    hideable: false,
                    width: 754,
                    renderer: this.defMainGridRenderer
                } ],
                tbar: [ this.oButtonNew, this.oButtonEdit,
                // { xtype: 'button', text: 'Erweiterte Ansicht',
                // id: 'action-selector-btn-extended' ,
                // disabled:true},
                    '->', this.lblSelectorAnzeige, this.oButtonUp, this.oButtonDown, this.oButtonDelete ]
            });
        }
        addDataIdToElement(this.mainGrid, this.actionSelectorName + 'Grid');
        this.mainGrid.parent = this;
        this.mainGrid.on({
            selectionchange: this.eventHandlerSelectionChange,
            edit: function (editor, eventObject) {
                var hrs = eventObject.record.get('delayHr');
                var min = eventObject.record.get('delayMin');
                var sec = eventObject.record.get('delaySec');
                eventObject.record.set('delay', hrs * dss.commonConsts.SECONDS_PER_HOUR + min * dss.commonConsts.SECONDS_PER_MINUTE + sec);
                mThis.gridStore.sort('delay', 'ASC');
            },
            itemdblclick: function () {     // grid, record, item, index, e
                mThis.oActualValue = mThis.getSelectedMainRecord().get('action');
                mThis.macPanel.expand(true);
            }
        });
        this.macPanel = Ext.create('Ext.panel.Panel', {
            border: '0',
            region: 'south',
            title: _('show activity editor'),
            collapsible: true,
            collapsed: false,
            titleCollapse: true,
            animCollapse: true,
            height: 320,
            layout: {
                type: 'hbox'
            },
            items: [ this.grids.lvl1 ],
            listeners: {
                beforecollapse: function (pPanel) {
                    pPanel.up().setMacGrid();
                    pPanel.setTitle('');
                },
                collapse: function (pPanel) {
                    pPanel.setTitle('');
                    mThis.oButtonEdit.setDisabled(pPanel.up().getSelectedMainRecord() == null);
                    mThis.oButtonEdit.setText(_('edit activity'));
                    mThis.doLayout();
                },
                expand: function (pPanel) {
                    pPanel.up().updatePanelDescription();
                    mThis.oButtonEdit.setDisabled(false);
                    mThis.oButtonEdit.setText(_('show all activities'));
                    mThis.doLayout();
                    mThis.setMacGrid(mThis.oActualValue);
                }
            },
            overflowX: 'scroll',
            overflowY: 'auto'
        });
        Ext.override(this.macPanel, {
            add: function () {
                this.callParent(arguments);
                this.body.dom.scrollLeft = 9999;    // scroll to the maximum right
            }
        });

        Ext.apply(this, {
            items: [ this.mainGrid, this.macPanel ]
        });
        defbug = this;
        this.callParent(arguments);
        this.on('updatePanelDescription', this._updatePanelDescription, this);
    },
    reset: function () {
        this.setValue(null);
    },
    tabChange: function () {    // fActive
        // eslint-disable-line no-empty-function
    },
    setValue: function (values) {
        this.grids.lvlDeviceScene && this.macPanel.remove(this.grids.lvlDeviceScene);
        this.grids.lvlDeviceAction && this.macPanel.remove(this.grids.lvlDeviceAction);
        this.grids.lvlDeviceActionSub && this.macPanel.remove(this.grids.lvlDeviceActionSub);
        this.grids.lvlRoom && this.macPanel.remove(this.grids.lvlRoom);
        this.grids.lvlHighLevelEvent && this.macPanel.remove(this.grids.lvlHighLevelEvent);
        this.grids.lvlURL && this.macPanel.remove(this.grids.lvlURL);
        this.grids.lvlGroup && this.macPanel.remove(this.grids.lvlGroup);
        this.grids.lvlScene && this.macPanel.remove(this.grids.lvlScene);
        this.grids.lvlDevice && this.macPanel.remove(this.grids.lvlDevice);
        this.grids.lvlUserGroup && this.macPanel.remove(this.grids.lvlUserGroup);
        this.grids.lvlUsergroupScene && this.macPanel.remove(this.grids.lvlUsergroupScene);
        this.grids.lvlUserState && this.macPanel.remove(this.grids.lvlUserState);
        this.grids.lvlAudio && this.macPanel.remove(this.grids.lvlAudio);
        // this.grids.lvlHeating && this.macPanel.remove(this.grids.lvlHeating);
        this.removeCls('myInvalid');
        for (var i = this.gridStore.getCount(); i > 0; i--) {
            try {
                this.gridStore.removeAt(i - 1);
            } catch (e) {
                devLog && devLog(e);
            }
        }
        var validatedValues = [];
        for (i = 0; values && i < values.length; i++) {
            if (dss.validateActionObject(values[i]) !== dss.validValue.completlyWrong) {
                validatedValues.push(values[i]);
            }
        }
        values = (validatedValues.length ? validatedValues : null);
        if (values == null) {
            this.doLayout();
            this.macPanel.expand(false);
            this.oButtonEdit.setText(_('show all activities'));
            this.lblSelectorAnzeige.setText(_('no activity'), false);
            this.setMacGrid();
        } else {
            for (i = 0; i < values.length; i++) {
                var oObject = {
                    type: values[i].type,
                    params: {}
                };
                var delay = 0;
                for (var sKey in values[i]) {
                    if (sKey !== 'type') {
                        oObject.params[sKey] = values[i][sKey];
                    }
                    if (sKey === 'delay') {
                        delay = Number(values[i][sKey]);
                    }
                }

                if (delay == null) {
                    delay = 0;
                }
                try {
                    this.gridStore.add({
                        action: oObject,
                        delay: delay,
                        delayHr: Math.floor(delay / dss.commonConsts.SECONDS_PER_HOUR),
                        delayMin: Math.floor(delay / dss.commonConsts.MINUTES_PER_HOUR) % dss.commonConsts.SECONDS_PER_MINUTE,
                        delaySec: delay % dss.commonConsts.SECONDS_PER_MINUTE
                    });
                } catch (e) {
                    devLog && devLog(e);
                }
            }
            this.setSelectedMainRecord(0);
            this.doLayout();
            if (values.length === 1) {
                this.oActualValue = this.getSelectedMainRecord().get('action');
                this.macPanel.expand(false);
                this.oButtonEdit.setText(_('show all activities'));
                this.setMacGrid(this.oActualValue);
            } else {
                this.macPanel.collapse(false);
                this.oButtonEdit.setText(_('edit activity'));
                this.setMacGrid();
            }
        }
    },
    isValid: function () {
        var valid = this.gridStore.getCount() > 0 || this.fAllowNoAction;
        this.gridStore.each(function (record) {
            if (record.get('action').type == null) {
                valid = false;
            } else if (record.get('action').type === 'undefined') {
                valid = false;
            } else if (record.get('action').type === DSS.ActionTypes.ZONE_VALUE) {
                valid = false;
            } else if (record.get('action').type === 'undo-zone-value') {
                valid = false;
            }
        });

        if (valid) {
            this.removeCls('myInvalid');
        } else {
            this.addCls('myInvalid');
        }
        return valid;
    },
    getValue: function () {
        var oResult = [];
        this.gridStore.each(function (record) {
            var oObject = {};
            var oRecord = record.get('action');
            var oDelay = record.get('delay');
            oObject.type = oRecord.type;
            for (var oKey in oRecord.params) {
                oObject[oKey] = oRecord.params[oKey];
            }
            if (oDelay != null) {
                oObject.delay = oDelay;
            } else {
                oObject.delay = 0;
            }
            oResult.push(oObject);
        });
        return oResult;
    }
}); // DSS.component.ActionConfigurator

var defbug;
